C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
C                   --------------------------------
                   SUBROUTINE EIGENN (W,VALU,VECT,NG)
C                   --------------------------------
C
      IMPLICIT NONE
      INTEGER *4 NG
      REAL    *4 W(3,3),VALU(3),VECT(3,3)
C
C
C     ***** EIGENVALUES AND EIGENVECTORS OF 3X3 MATRIX *****
C
      INTEGER *4 I,J,II,III,I1,IMAX,L,LLL
      REAL    *4 A(3,3),B(3,3),V(3),U(3),ERRND,SIGMA,TEM,B0,B1,B2
      REAL    *4 VOLD,VNEW,X,Y,C0,C1,TEMP,T1,SMAX,VMV
C
C     ***** START OF PROGRAM *****
C
      ERRND=5.E-7
      SIGMA=0.
      DO  J=1,3
        DO  I=1,3
          TEM=W(I,J)
          A(I,J)=TEM
          SIGMA=SIGMA+TEM*TEM
        ENDDO
      ENDDO
C
C     ***** CHECK FOR NULL MATRIX *****
C
      IF (SIGMA .LE. 0.) THEN
        NG=6
        RETURN
      ENDIF
      SIGMA=SQRT(SIGMA)
C
C     ***** FORM CHARACTERISTIC EQUATION *****
C
      B2=A(1,1)+A(2,2)+A(3,3)
      B1=-A(1,1)*A(2,2)-A(1,1)*A(3,3)-A(2,2)*A(3,3)+A(1,3)*A(3,1)
     1+A(2,3)*A(3,2)+A(1,2)*A(2,1)
      B0=A(1,1)*A(2,2)*A(3,3)+A(1,2)*A(2,3)*A(3,1)+A(1,3)*A(3,2)*A(2,1)-
     1A(1,3)*A(3,1)*A(2,2)-A(1,1)*A(2,3)*A(3,2)-A(1,2)*A(2,1)*A(3,3)
C
C     ***** FIRST ROOT BY BISECTION *****
C
      X=0.
      Y=SIGMA
      TEM=((B2-SIGMA)*SIGMA+B1)*SIGMA+B0
      VNEW=0.0
      IF (B0 .EQ. 0.) GO TO 10
      IF (B0 .LT. 0.) THEN
        IF (TEM .LE. 0.) Y=-Y
      ELSE
        Y=0.
        X=SIGMA
        IF (TEM .GT. 0.) X=-X
      ENDIF
C
C     ***** NOW PHIF(X).LT.0.AND.PHIF(Y).GT.0. *****
C
      VNEW=(X+Y)*.5
      DO  I=1,40
        TEMP=((B2-VNEW)*VNEW+B1)*VNEW+B0
        IF (TEMP .EQ. 0) GO TO 10
        IF (TEMP .LT. 0) THEN
          X=VNEW
        ELSE
          Y=VNEW
        ENDIF
        VOLD=VNEW
        VNEW=(X+Y)*.5
        TEM=ABS(VOLD-VNEW)
        IF (TEM .LE. ERRND) GO TO 10
        IF (VOLD .NE. 0.) THEN
          IF (ABS(TEM/VOLD) .LE. ERRND) GO TO 10
        ENDIF
      ENDDO
C
C     ***** DID NOT CONVERGE, SET ERROR INDICATOR *****
C
      NG=6
      RETURN
C
C     ***** STORE FIRST ROOT *****
C
   10 U(3)=VNEW
C
C     ***** DEFLATE *****
C
      C1=B2-VNEW
      C0=B1+C1*VNEW
C
C     ***** SOLVE QUADRATIC *****
C
      TEM=C1*C1+4.*C0
      IF (TEM .LE. 0) THEN
C
C     ***** IGNORE IMAGINARY COMPONENT OF COMPLEX ROOT *****
C
        TEM=0.
      ELSE
        TEM=SQRT(TEM)
      ENDIF
      U(1)=.5*(C1-TEM)
      U(2)=.5*(C1+TEM)
C
C     ***** SORT ROOTS *****
C
      DO  J=1,2
        IF (U(J) .GT. U(3)) THEN
          TEM=U(J)
          U(J)=U(3)
          U(3)=TEM
        ENDIF
      ENDDO
      LLL=-2
      DO  III=1,2
C
C       ***** CHECK FOR MULTIPLE ROOTS *****
C
        TEM=ERRND*100.
        NG=0
        L=1
        DO  I=1,2
          IF ((U(I+1)-U(I) .LE. TEM) .OR.
     +    ((U(I) .NE. 0.).AND.(ABS((U(I+1)-U(I))/U(I)) .LE. TEM))) THEN
            L=L-1
            NG=NG-2*I
          ENDIF
        ENDDO
        IF (LLL .GE. L) RETURN
        LLL=L
C
C       ***** EIGENVECTOR ROUTINE *****
C
        DO  II=1,3
          T1=U(II)
          IF ((L .LT. 0) .OR. ((L .EQ. 0) .AND. (NG+5 .NE. II))) THEN
C
C         ***** TWO VECTORS NULL FOR DOUBLE ROOT *****
C         ***** ALL VECTORS NULL FOR TRIPLE ROOT *****
C
            DO  J=1,3
              VECT(J,II)=0.0
            ENDDO
            GO TO 20
          ENDIF
          DO  J=1,3
            A(J,J)=W(J,J)-T1
          ENDDO
          SMAX=0.0
          DO  I=1,3
            I1=1
            IF (I .LE. 2) I1=I+1
            B(I,1)=A(I,2)*A(I1,3)-A(I,3)*A(I1,2)
            B(I,2)=A(I,3)*A(I1,1)-A(I,1)*A(I1,3)
            B(I,3)=A(I,1)*A(I1,2)-A(I,2)*A(I1,1)
            TEM=B(I,1)**2+B(I,2)**2+B(I,3)**2
            IF (TEM .GT. SMAX) THEN
              SMAX=TEM
              IMAX=I
            ENDIF
          ENDDO
          IF (SMAX .LE. 0.) THEN
            NG=7
            GO TO 20
          ENDIF
          SMAX=SQRT(SMAX)
          DO J=1,3
            V(J)=B(IMAX,J)/SMAX
          ENDDO
C
C         ***** REFINE EIGENVECTOR *****
C
          CALL AXEQB (A,V,V,1)
          TEM=AMAX1(ABS(V(1)),ABS(V(2)),ABS(V(3)))
          DO  J=1,3
            V(J)=V(J)/TEM
          ENDDO
          CALL UNIT (V,VECT(1,II),1)
C
C         ***** REFINE EIGENVALUE *****
C
          T1=VMV(VECT(1,II),W,VECT(1,II))
          U(II)=T1
   20     VALU(II)=T1
        ENDDO
      ENDDO
      RETURN
      END
