// KPLOT - A Program for Plotting and Analyzing Crystal Structures 
// Copyright (C) 1979-2019 Rudolf Hundt 
// KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation, version 3.
// KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
// See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see https://www.gnu.org/licenses/
// Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
// Co-Programmer, mailto:k.pleuger@t-online.de


#include "kplot.h"

#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilinedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qtabbar.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qlabel.h>
#include <qtimer.h>
//#include <glib.h>
#include <qheader.h>
//#include <qwindow.h>
#include <qlineedit.h>

#include <fstream.h>
//#include "neu.xpm"

#include "FM.xpm"
#include "FZ.xpm"
#include "GNZL.xpm"
#include "LCFF.xpm"
#include "fileopen.xpm"
#include "filesave.xpm"
#include "fileprint.xpm"
#include "viewmag+.xpm"
#include "viewmag-.xpm"
#include "viewmag.xpm"
#include "epu.xpm"
#include "stpu.xpm"
#include "filenew2.xpm"
#include "c60.xpm"
#include "viewmag3d+.xpm"
#include "viewmag3d-.xpm"

#include "kplot2d.h"
#include "kplot3d.h"
#include "wood.h"
#include "metal.h"
#include <qplatinumstyle.h>

extern void strlower(char *);
extern "C" void kpmain_();

const char * fileOpenText = "Dr�cke diese Taste um eine neue KPlot Datei zu laden.\n\n"
"Sie k�nnen auch dieses Kommando aus dem Men� aktivieren.";
/*
const char * fileSaveText = "Click this button to save the file you are "
"editing.  You will be prompted for a file name.\n\n"
"You can also select the Save command from the File menu.\n\n"
"Note that implementing this function is left as an exercise for the reader.";
const char * filePrintText = "Click this button to print the file you "
"are editing.\n\n"
"You can also select the Print command from the File menu.";
*/

#define MAX_WARN_ENTRIES 600

void CKplotApp::output(QString text,int flag)
{
/*   cout << "Output" << endl; */
   if (!flag)
   {
   	outputList->insertItem(text);
   }
   else
   {
        outputList->changeItem(outputList->text(outputList->count()-1)+text,outputList->count()-1);
   }
   if (outputList->count()>MAX_WARN_ENTRIES)
   {          
      outputList->removeItem(0);
   }
   outputList->setTopItem(outputList->count()-1);
   outputList->repaint();
}

QString CKplotApp::getInput()
{
   return QString(lineEdit->text());
}

void CKplotApp::resetInput()
{
   lineEdit->setText("");
}

void CKplotApp::processDrawCommand(float *ar , int *funcIn)
{
   static long int state=0;

   if (*funcIn==32)
   {
      state=*((/*long*/ int*)(&(ar[0])));   
      if (state<0)
		 emit opt3D();
      else   
		 emit opt2D();
   }
   if (state<0)
   {
/*
   if (flag==-1)
   {
   if (*funcIn==0)
   {
      wgl->EndDraw();
   }
   if (*funcIn==6)
   {
      wgl->BeginDraw();
   }
*/
     wgl->processDrawCommand(ar,funcIn);
   }
   else
   {
     kplot2d->processDrawCommand(ar,funcIn);
   }
}


CKplotApp::CKplotApp()
    : QMainWindow( 0, "example application main window" )
{
//    QPixmap neu;
	interruptFlag=FALSE;
    ToolBarDock edge=Top;
    ToolBarDock top=Top;
    setDockEnabled(edge,TRUE);
    fileTools = new QToolBar( this, "file operations" );
//    removeToolBar(fileTools);
    addToolBar(fileTools,"file operations",top);
    tools3d = new QToolBar(this,"3D Options");
//    removeToolBar(tools3d);
    addToolBar(tools3d,"3D Options",top);	
    tools = new QToolBar(this,"Tools");
//    removeToolBar(tools);
    addToolBar(tools,"Tools",top);	

	QWidget *w=new QWidget(this);
    layout=new QGridLayout(w,2,1);
	
	
    split=new QSplitter(w);
	split->setOrientation(QSplitter::Vertical);

	
	w2d=new QWidget(split);
        QGridLayout *gl=new QGridLayout(w2d,1,1);
	kplot2d=new CKPlot2D(w2d,"Kplot Graphics");
        gl->addWidget(kplot2d,0,0);
        w3d=new QWidget(split);
        QGridLayout *gl2=new QGridLayout(w3d,1,1);
    wgl=new CKPlot3D(w3d,"OpenGl");
        gl2->addWidget(wgl,0,0);
    if ( !wgl->isValid() ) {
	// Try without double-buffering
	QGLFormat fmt( FALSE );
	wgl->setFormat( fmt );
	if ( !wgl->isValid() )
	    fatal("Failed to create OpenGL rendering context on this display");
    }
	w3d->hide();
	w2d->show();
	layout->activate();


	outputList=new QListBox(split,"Output List");
	layout->addWidget(split,0,0);
//	outputList->setFixedVisibleLines(4);
	lineEdit=new QLineEdit(w);	
	lineEdit->setMinimumHeight(lineEdit->sizeHint().height());
	layout->addWidget(lineEdit,1,0);
	layout->setRowStretch(0,10);
	
	
	setCentralWidget(w);
	layout->activate();

	
//    neu = QPixmap( new_xpm );
    QPixmap loadPixmap = QPixmap(fileopen);
    QPixmap savePixmap = QPixmap(filesave);
    QPixmap printPixmap = QPixmap(fileprint);
    QPixmap zoomBestPixmap = QPixmap(viewmagBest);
    QPixmap zoomPlusPixmap = QPixmap(viewmag);
    QPixmap zoomMinusPixmap = QPixmap(viewmagMinus);
    QPixmap epuPixmap = QPixmap(epuPix);
    QPixmap stpuPixmap = QPixmap(stpuPix);
	QPixmap newPixmap = QPixmap(filenew);
	c60Pixmap=new QPixmap(c60);
    QPixmap zoomPlusPixmap3d = QPixmap(viewmag3d);
    QPixmap zoomMinusPixmap3d = QPixmap(viewmag3dMinus);
 	
    QApplication::setFont(QFont("courier",18),FALSE);
    QToolButton * newFilePixmap = new QToolButton( newPixmap, "New", QString::null,
					      this, SLOT(restart()),
					      fileTools, "New (Restart)" );
    QToolButton * fileOpen = new QToolButton( loadPixmap, "Laden", QString::null,
					      this, SLOT(load()),
					      fileTools, "KPlot Datei laden." );

    QToolButton * fileSave = new QToolButton( savePixmap, "Speichern", QString::null,
					      this, SLOT(save()),
					      fileTools, "KPlot Datei speichern." );

    QToolButton * filePrint = new QToolButton( printPixmap, "Drucken", QString::null,
					      kplot2d, SLOT(print()),
					      fileTools, "KPlot Grafik drucken." );

    QToolButton * epu = new QToolButton( epuPixmap, "EPU", QString::null,
					      this, SLOT(epuCommand()),
					      fileTools, "EPU" );
    QToolButton * stpu = new QToolButton( stpuPixmap, "STPU", QString::null,
					      this, SLOT(stpuCommand()),
					      fileTools, "STPU" );

    QToolButton * zoomBest = new QToolButton( zoomBestPixmap, "Best fit", QString::null,
					      this, SLOT(zoomBest()),
					      fileTools, "Best fit" );
    QToolButton * zoomPlus = new QToolButton( zoomPlusPixmap, "Double field", QString::null,
					      this, SLOT(zoomPlus()),
					      fileTools, "Zoom + x2" );
    QToolButton * zoomMinus = new QToolButton( zoomMinusPixmap, "Half field", QString::null,
					      this, SLOT(zoomMinus()),
					      fileTools, "Zoom - x0.5" );

    opt2DTool = new QToolButton( zoomMinusPixmap3d,
              "3D Zoom +", QString::null, 
	this,                 
         SLOT(opt2D()),tools3d, "Z-Pos erniedrigen" );   

    opt3DTool = new QToolButton( zoomPlusPixmap3d, 
          "3D Zoom -", QString::null,
					      this, SLOT(opt3D()),
					      tools3d, "Z-Pos erh�hen" );
opt2DTool->setToggleButton(TRUE);
opt3DTool->setToggleButton(TRUE);
opt3DTool->setOn(FALSE);
opt2DTool->setOn(TRUE);


    QPixmap FMPixmap = QPixmap(FM_xpm);
    QToolButton * FMButtonPixmap = new QToolButton( FMPixmap, "Finde Molek�l (FM)", QString::null,
					      this, SLOT(FMKommando()),
					      tools, "FM" );

    QPixmap FZPixmap = QPixmap(FZ_xpm);
    QToolButton * FZButtonPixmap = new QToolButton( FZPixmap, "Finde Zelle (FZ)", QString::null,
					      this, SLOT(FZKommando()),
					      tools, "FZ" );

    QPixmap GNZLPixmap = QPixmap(GNZL_xpm);
    QToolButton * GNZLButtonPixmap = new QToolButton( GNZLPixmap, "Generiere zelle (GNZL)", QString::null,
					      this, SLOT(GNZLKommando()),
					      tools, "GNZL" );

    QPixmap LCFFPixmap = QPixmap(LCFF_xpm);
    QToolButton * LCFFButtonPixmap = new QToolButton( LCFFPixmap, "N�chster Entry (LCFF)", QString::null,
					      this, SLOT(LCFFKommando()),
					      tools, "LCFF" );

//	fileOpen->setUsesTextLabel(TRUE);				      
//	fileSave->setUsesTextLabel(TRUE);				      
//	fileOpen->setUsesTextLabel(TRUE);				      
//	filePrint->setUsesTextLabel(TRUE);				      

    QApplication::setFont(QFont("courier",18),FALSE);
	
    (void)QWhatsThis::whatsThisButton( fileTools );
    QWhatsThis::add( fileOpen, fileOpenText);//, FALSE );
    
    QPopupMenu * file = new QPopupMenu();
    menuBar()->insertItem( "&Datei", file );

    file->insertItem( "Laden ...", this, SLOT(load()), CTRL+Key_L );
    file->insertItem( "Speichern ...", this, SLOT(save()), CTRL+Key_S );
    QPopupMenu * exp=new QPopupMenu();
    file->insertItem("&Export",exp);
    exp->insertItem( "BMP", kplot2d, SLOT(exportBMP()), CTRL+Key_B );
    exp->insertItem( "XPM", kplot2d, SLOT(exportXPM()), CTRL+Key_X );
    file->insertSeparator();
    file->insertItem( "Drucken ...", kplot2d, SLOT(print()), CTRL+Key_D );
    file->insertItem( "Beenden", qApp, SLOT(quit()), CTRL+Key_Q );

/*	
	aktionMenu = new QPopupMenu();
	menuBar()->insertItem("&Test4",aktionMenu);
	
	aktionMenu->insertItem("&T1",this,0,0);
	aktionMenu->insertItem("&T2",this,0,0);
	aktionMenu->insertSeparator();
	aktionMenu->insertItem("&T3",this,0,0);
	aktionMenu->insertItem("&T4",this,0,0);
	aktionMenu->insertItem("&T5",this,0,0);
	aktionMenu->insertItem("&T6",this,0,0);
	aktionMenu->insertItem("&T7",this,0,0);
	aktionMenu->insertItem("&T8",this,0,0);
	aktionMenu->insertSeparator();
	aktionMenu->insertItem("&T9",this,0,0);
	aktionMenu->insertItem("&T10",this,0,0);
*/	
    options = new QPopupMenu();
    menuBar()->insertItem( "&Optionen", options );
    bufferedDraw = options->insertItem( "gepuffertes Malen", this, SLOT(toggleBufferedDraw()), CTRL+Key_P);
    backgroundBlack = options->insertItem( "Hintergrund schwarz", this, SLOT(toggleBackgroundColor()), CTRL+Key_S);
    QPopupMenu *options2 = new QPopupMenu();
    options->insertItem("&3D Optionen",options2);
    rotate3D = options2->insertItem( "Rotation 3D", this,
SLOT(toggleRotate3D()), CTRL+Key_R);
    solid3D = options2->insertItem("Solid Mode",this,
    SLOT(toggleSolid3D()),CTRL+Key_S);
    renderMode = options2->insertItem( "Better Rendering", this,
SLOT(toggleRenderMode()), CTRL+Key_B);
	renderPixmapMenu=new QPopupMenu();
	options2->insertItem("Render pixmap",renderPixmapMenu);
    renderNonePixmap = renderPixmapMenu->insertItem( "None", this,
SLOT(toggleRenderNonePixmap()));
    renderMarmorPixmap = renderPixmapMenu->insertItem( "Marmor", this,
SLOT(toggleRenderMarmorPixmap()));
	renderPixmapMenu->setCheckable(TRUE);
	renderPixmapMenu->setItemChecked(renderNonePixmap,TRUE);
/*    options2->insertItem( "Zoom 3D +", this,
SLOT(optZPosMinus()), CTRL+Key_Plus);
    options2->insertItem( "Zoom 3D -", this,
SLOT(optZPosPlus()), CTRL+Key_Minus);*/
    options2->insertItem( "Reset positions",this,
     SLOT(resetPos3D()),0);
    options2->insertItem( "Save as Postscript ...", this,
SLOT(print3D()), CTRL+Key_P);
     options->setCheckable(TRUE);    
     options2->setCheckable(TRUE);

    options->setItemChecked( bufferedDraw,TRUE);    
    kplot2d->setBufferedMode(TRUE);
    options->setItemChecked(backgroundBlack,TRUE);
    wgl->setRotate3D(FALSE);
    options2->setItemChecked( rotate3D, FALSE);
    wgl->setSolidMode(TRUE);
    options2->setItemChecked( solid3D, TRUE);
    wgl->setRenderMode(FALSE);
    options2->setItemChecked( renderMode, FALSE);
	
    controls = new QPopupMenu();
    menuBar()->insertItem( "&Controls", controls );

    QAccel* a = new QAccel(this);
    a->connectItem( a->insertItem( CTRL+Key_M ), this, SLOT(toggleMenuBar()) );

    tb = controls->insertItem( "Tool bar", this, SLOT(toggleToolBar()), CTRL+Key_T);
    sb = controls->insertItem( "Status bar", this, SLOT(toggleStatusBar()), CTRL+Key_S);
    controls->setCheckable( TRUE );
    controls->setItemChecked( mb, TRUE );
    controls->setItemChecked( tb, TRUE );
    controls->setItemChecked( sb, TRUE );
	controls->setCheckable(TRUE);    

    styles = new QPopupMenu();

    menuBar()->insertItem( "&Window Styles", styles );
    tWindowsStyle = styles->insertItem( "Windows Style", this, SLOT(toggleWindowsStyle()), CTRL+Key_W);
    tMotifStyle = styles->insertItem( "Motif Style", this, SLOT(toggleMotifStyle()), CTRL+Key_M);
    tPlatinumStyle = styles->insertItem( "Platinum Style", this, SLOT(togglePlatinumStyle()), CTRL+Key_P);
    tWoodStyle = styles->insertItem( "Wood Style", this, SLOT(toggleWoodStyle()), CTRL+Key_W);
    tMetalStyle = styles->insertItem( "Metal Style", this, SLOT(toggleMetalStyle()), CTRL+Key_E);
	styles->setCheckable(TRUE);    
	
    hilfeMenu = new QPopupMenu;
    menuBar()->insertItem ("&Hilfe", hilfeMenu);
	hilfeMenu->insertItem("&Info ...",this,SLOT(about()),CTRL+Key_I);
    

	statusBar()->addWidget(new QLabel("",statusBar(),"zeit"),40,TRUE);
	statusBar()->addWidget((zeit=new QLabel("12:09:1998",statusBar(),"zeit")),10,TRUE);
    statusBar()->message( "Ready", 2000 );
	

		
QTimer *timer=new QTimer(this);
connect(timer,SIGNAL(timeout()),this,SLOT(timeout()));
timer->start(1000,FALSE);
resize(800,600);
split->setResizeMode(w2d,QSplitter::Stretch);
split->setResizeMode(w3d,QSplitter::Stretch);
connect(lineEdit,SIGNAL(returnPressed()),this,SLOT(input()));
connect(kplot2d,SIGNAL(interrupt()),this,SLOT(interrupt()));
QValueList<int> vl=split->sizes();
vl[0]=400;
vl[1]=400;
split->setSizes(vl);


	styles->setItemChecked( tWindowsStyle, TRUE );	
    qApp->setStyle(new QWindowsStyle());

}

void CKplotApp::interrupt()
{
   interruptFlag=TRUE; 
}

void CKplotApp::zoomPlus()
{
	QString s="df;rdrw";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::restart()
{
	QString s="rstr,";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::resetPos3D()
{
        wgl->Reset();
}

void CKplotApp::zoomMinus()
{
	QString s="hf;rdrw";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::zoomBest()
{
	QString s="bf;rdrw";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::FMKommando()
{
	QString s="fm;";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::FZKommando()
{
	QString s="fz;";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::GNZLKommando()
{
	QString s="gnzl;";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::LCFFKommando()
{
	QString s="lcff;";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::epuCommand()
{
	QString s="epu";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::stpuCommand()
{
	QString s="stpu";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::print3D()
{
   QString f=QFileDialog::getSaveFileName(QString::null,"*.eps",this);
   if (!f.isEmpty())
   {
	  QString s=f;
          wgl->outputPS(s);
//	  cout << "Save Postscript" << endl;
   }
}


void CKplotApp::input(void)
{
   qApp->exit(1);	
/* cout << "Input" << endl; */
}

void CKplotApp::load(void)
{
   QString f=QFileDialog::getOpenFileName(QString::null,"*.kpl",this);
   if (!f.isEmpty())
   {
	  QString s="get '";
	  s=s+f;
	  s=s+"'";
	  lineEdit->setText(s);
	  qApp->exit(1);	
/*	  cout << "Load" << endl; */
   }
}

void CKplotApp::save(void)
{
   QString f=QFileDialog::getSaveFileName(QString::null,"*.kpl",this);
   if (!f.isEmpty())
   {
	  QString s="putc '";
	  s=s+f;
	  s=s+"'";
	  lineEdit->setText(s);
	  qApp->exit(1);	
/*	  cout << "Save" << endl; */
   }
}


void CKplotApp::timeout(void)
{
//  fehler_cls.f_warning("Timer done",__FILE__,__LINE__);
  QTime time=QTime::currentTime();
  QString str=time.toString();
  zeit->setText(str);  
//  kpmain_(); 
  
}
// setUpLayout();
/*! Destroys the object and frees any allocated resources.

*/

CKplotApp::~CKplotApp()
{
    // no explicit destruction is necessary in this example
//    delete berTabelle;
}





void CKplotApp::about(void)
{
   QMessageBox messageBox(this);
   messageBox.setCaption("About KPLOT");
   messageBox.setText(QString("   KPLOT"
				 "\nby R. Hundt\n"));
   messageBox.setIconPixmap(*c60Pixmap);
   messageBox.exec();				 
/*   QMessageBox::about(this,"About KPLOT",QString("   KPLOT"
				 "\nby R. Hundt\n"
				 ));
*/
				 
}

void CKplotApp::toggleMenuBar()
{
    if ( menuBar()->isVisible() ) {
	menuBar()->hide();
	controls->setItemChecked( mb, FALSE );
    } else {
	menuBar()->show();
	controls->setItemChecked( mb, TRUE );
    }
}

void CKplotApp::toggleToolBar()
{
    if ( fileTools->isVisible() ) {
	fileTools->hide();
	tools3d->hide();
	controls->setItemChecked( tb, FALSE );
    } else {
	fileTools->show();
	tools3d->show();
	controls->setItemChecked( tb, TRUE );
    }
}

void CKplotApp::toggleBufferedDraw()
{
    if ( options->isItemChecked(bufferedDraw)) {
	  kplot2d->setBufferedMode(FALSE);
	  wgl->setBufferedMode(FALSE);	  
	  options->setItemChecked( bufferedDraw, FALSE );
    } else {
	  kplot2d->setBufferedMode(TRUE);
	  wgl->setBufferedMode(TRUE);
	  options->setItemChecked( bufferedDraw, TRUE );
    }
}

void CKplotApp::toggleRotate3D()
{
    if ( options->isItemChecked(rotate3D)) {
	  wgl->setRotate3D(FALSE);
	  options->setItemChecked( rotate3D, FALSE );
    } else {
	  wgl->setRotate3D(TRUE);
	  options->setItemChecked( rotate3D, TRUE );
    }
}

void CKplotApp::toggleSolid3D()
{
    if ( options->isItemChecked(solid3D)) {
	  wgl->setSolidMode(FALSE);
	  options->setItemChecked( solid3D, FALSE );
    } else {
	  wgl->setSolidMode(TRUE);
	  options->setItemChecked( solid3D, TRUE );
    }
}

void CKplotApp::toggleRenderMode()
{
    if ( options->isItemChecked(renderMode)) {
	  wgl->setRenderMode(FALSE);
	  options->setItemChecked( renderMode, FALSE );
    } else {
	  wgl->setRenderMode(TRUE);
	  options->setItemChecked( renderMode, TRUE );
    }
}

void CKplotApp::toggleBackgroundColor()
{
    if ( options->isItemChecked(backgroundBlack)) {
	  kplot2d->changeBackgroundColor();
	  wgl->changeBackgroundColor();
	  options->setItemChecked( backgroundBlack, FALSE );
    } else {
	  kplot2d->changeBackgroundColor();
	  wgl->changeBackgroundColor();
	  options->setItemChecked( backgroundBlack, TRUE );
    }
}

void CKplotApp::toggleStatusBar()
{
    if ( statusBar()->isVisible() ) {
	statusBar()->hide();
	controls->setItemChecked( sb, FALSE );
    } else {
	statusBar()->show();
	controls->setItemChecked( sb, TRUE );
    }
}

void CKplotApp::opt2D()
{
QValueList<int> vl=split->sizes();
//	w3d->hide();
//	layout2->addWidget(kplot2d,0,0);
vl[0]=vl[1];
/*int v=vl[0];
vl[0]=vl[1];
vl[1]=v;*/
//cout << vl[0] << " " << vl[1] << endl;
	w2d->show();
split->setSizes(vl);
//	w2d->show();
	w3d->hide();
	layout->activate();
        opt3DTool->setOn(FALSE);
        opt2DTool->setOn(TRUE);
//    wgl->incrementZPos();
	QString s="ortp 0;rdrw";
	lineEdit->setText(s);
	qApp->exit(1);
}

void CKplotApp::opt3D()
{
QValueList<int> vl=split->sizes();
//	w2d->hide();
vl[1]=vl[0];
/*int v=vl[0];
vl[0]=vl[1];
vl[1]=v;*/
	w3d->show();
//split->setSizes(vl);
//	layout2->addWidget(wgl,0,0);
//	w3d->show();
	w2d->hide();
	layout->activate();
        opt2DTool->setOn(FALSE);
        opt3DTool->setOn(TRUE);
//    wgl->decrementZPos();
	QString s="ortp -1;rdrw";
	lineEdit->setText(s);
	qApp->exit(1);
}

/*
void CKplotApp::SetAtom3D(float *x, float* y, float * z, float * r, long int *
nr, long int * color) 
{
 wgl->SetAtom3D(x,y,z,r,nr,color);
}

void CKplotApp::SetBond3D(long int * p1, long int * p2, long int * color,
float * radius, float * zPos) {
 wgl->SetBond3D(p1,p2,color,radius,zPos);
}
*/

void CKplotApp::toggleWindowsStyle()
{
	  styles->setItemChecked( tWindowsStyle, TRUE );
	  styles->setItemChecked( tMotifStyle, FALSE );
	  styles->setItemChecked( tPlatinumStyle, FALSE );
	  styles->setItemChecked( tWoodStyle, FALSE );
	  styles->setItemChecked( tMetalStyle, FALSE );
    qApp->setStyle(new QWindowsStyle);

}

void CKplotApp::toggleMotifStyle()
{
	  styles->setItemChecked( tWindowsStyle, FALSE );
	  styles->setItemChecked( tMotifStyle, TRUE );
	  styles->setItemChecked( tPlatinumStyle, FALSE );
	  styles->setItemChecked( tWoodStyle, FALSE );
	  styles->setItemChecked( tMetalStyle, FALSE );
    qApp->setStyle(new QMotifStyle);

}
void CKplotApp::togglePlatinumStyle()
{
	  styles->setItemChecked( tWindowsStyle, FALSE );
	  styles->setItemChecked( tMotifStyle, FALSE );
	  styles->setItemChecked( tPlatinumStyle, TRUE );
	  styles->setItemChecked( tWoodStyle, FALSE );
	  styles->setItemChecked( tMetalStyle, FALSE );
    qApp->setStyle(new QPlatinumStyle);

}
void CKplotApp::toggleWoodStyle()
{
	  styles->setItemChecked( tWindowsStyle, FALSE );
	  styles->setItemChecked( tMotifStyle, FALSE );
	  styles->setItemChecked( tPlatinumStyle, FALSE );
	  styles->setItemChecked( tWoodStyle, TRUE );
	  styles->setItemChecked( tMetalStyle, FALSE );
//    qApp->setStyle(new NorwegianWoodStyle);

}
void CKplotApp::toggleMetalStyle()
{
	  styles->setItemChecked( tWindowsStyle, FALSE );
	  styles->setItemChecked( tMotifStyle, FALSE );
	  styles->setItemChecked( tPlatinumStyle, FALSE );
	  styles->setItemChecked( tWoodStyle, FALSE );
	  styles->setItemChecked( tMetalStyle, TRUE );
//    qApp->setStyle(new MetalStyle);

}

void CKplotApp::toggleRenderNonePixmap()
{
	  renderPixmapMenu->setItemChecked( renderNonePixmap, TRUE );
	  renderPixmapMenu->setItemChecked( renderMarmorPixmap, FALSE );
	  wgl->RenderPixmap(0);
}

void CKplotApp::toggleRenderMarmorPixmap()
{
	  renderPixmapMenu->setItemChecked( renderNonePixmap, FALSE );
	  renderPixmapMenu->setItemChecked( renderMarmorPixmap, TRUE );
	  wgl->RenderPixmap(1);
}
