// KPLOT - A Program for Plotting and Analyzing Crystal Structures 
// Copyright (C) 1979-2019 Rudolf Hundt 
// KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation, version 3.
// KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
// See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see https://www.gnu.org/licenses/
// Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
// Co-Programmer, mailto:k.pleuger@t-online.de


#ifndef APPLICATION_H
#define APPLICATION_H

#include <qmainwindow.h>
#include "qstring.h"

class QMultiLineEdit;
class QToolBar;
class QPopupMenu;
class QListView;
class QListBox;
class QLabel;
class QGridLayout;
class QSplitter;
class QWidget;
class CKPlot2D;
class QLineEdit;
class CKPlot3D;
class QToolButton;
class CKplotApp: public QMainWindow
{
    Q_OBJECT
public:
    CKplotApp();
    ~CKplotApp();

public:
	void output(QString text,int flag=0);
    QString getInput();
    void resetInput();
    void processDrawCommand(float *ar , int *funcIn);
/*
    void SetAtom3D(float *x, float* y, float * z, float * r, long int *nr,
long int *color);
    void SetBond3D(long int * p1, long int * p2, long int * color, float
*radius, float * zPos);
*/
     bool interruptFlag;

public slots:
	void interrupt();   
	
private slots:
    void about();
	void input();
    void toggleMenuBar();
    void toggleStatusBar();
    void toggleToolBar();
    void toggleBufferedDraw();
    void toggleBackgroundColor();
    void toggleRotate3D();
    void toggleSolid3D();
    void toggleRenderMode();
    void timeout();
    void load();
    void save();
    void zoomMinus();
    void zoomPlus();
    void zoomBest(); 
    void epuCommand();
    void stpuCommand();
    void FMKommando();
    void FZKommando();
    void GNZLKommando();
    void LCFFKommando();
    void restart();
    void opt2D();
    void opt3D();
    void resetPos3D();
    void print3D();
    void toggleWindowsStyle();
    void toggleMotifStyle();
    void togglePlatinumStyle();
    void toggleWoodStyle();
    void toggleMetalStyle();
	void toggleRenderMarmorPixmap();
    void toggleRenderNonePixmap();

signals:
private:
	CKPlot2D *kplot2d;
	QLabel *zeit;
    QToolBar *fileTools;
    QPopupMenu *controls;
    QPopupMenu *options;
    QPopupMenu *aktionMenu;
    QPopupMenu *hilfeMenu;
    QPopupMenu * styles;
    QPopupMenu * renderPixmapMenu;
    QListBox   *outputList;
    QWidget *w2d,*w3d;
    int mb, tb, sb;
    int bufferedDraw;
    int backgroundBlack;
    int rotate3D;
    int solid3D;
    int renderMode;
	QGridLayout *layout;    
	QGridLayout *layout2;    
	QSplitter *split;
	QLineEdit *lineEdit;
	QPixmap * c60Pixmap;
	CKPlot3D *wgl;
        QToolButton * opt3DTool;
        QToolButton * opt2DTool;
	QToolBar * tools3d;	
	QToolBar * tools;	
    int tWindowsStyle;
    int tMotifStyle;
    int tPlatinumStyle;
    int tWoodStyle;
    int tMetalStyle;
    int renderMarmorPixmap;
    int renderNonePixmap;
    

};


#endif
