// KPLOT - A Program for Plotting and Analyzing Crystal Structures 
// Copyright (C) 1979-2019 Rudolf Hundt 
// KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation, version 3.
// KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
// See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see https://www.gnu.org/licenses/
// Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
// Co-Programmer, mailto:k.pleuger@t-online.de

//-*-Mode: C++;-*-
#ifndef _kplot2d_h_
#define _kplot2d_h_


#include <qframe.h>
#include <qwidget.h>
#include <qpointarray.h>
#include <qpen.h>
#include <qbrush.h>
#include <qpixmap.h>
#include <qcursor.h>

class QPrinter;

//---- newclass -----------------------------------------------------------

class CKPlot2D : public QFrame
{
	Q_OBJECT
	
public:

	enum {
	   ForeGroundIndex=15,
	   BackGroundIndex=0
	};
	
    CKPlot2D(QWidget *parent=0,const char *name=0);
    ~CKPlot2D();

public:
    void processDrawCommand(float *ar , int *funcIn);
	void setBufferedMode(bool flag){bufferedMode=flag;}
	void changeBackgroundColor();
    static QColor *color[16];

public slots:
	void print();
	void exportBMP();
	void exportXPM();

signals:
	void interrupt();

private:
    void paintEvent( QPaintEvent * );
    void setDefaultColors();
    
    QPainter *p;
    QPrinter *prt;
	
	QPointArray pointArray;
	QPen pen;
	QBrush brush;
	QPixmap offScreenBuffer;
	QPixmap exportBuffer;
	
	int polygon_perimeter;
 
	float linie_x;
	float linie_y;

	float x_cm;
	float y_cm;
	int   x_pixel; 	
	int   y_pixel;
	float x_ur,y_ur;



	float x_fak;
	float y_fak;
	float aktX,aktY;
	int f_color;
	
	bool bufferedMode;
	bool printing;
	bool holeCoord;
	bool exporting;
	QCursor cursorCross;
	QCursor cursorArrow;
	QString exportFile;		
	QString exportType;
protected:
    void mousePressEvent(QMouseEvent * ev);	

/*
    void resizeEvent( QResizeEvent * ){};
*/
};

#endif
