C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
      SUBROUTINE ludcmp (a,n,np,indx,d)
      IMPLICIT NONE
      INTEGER *4 n,np,indx(n),NMAX
      REAL    *4 d,a(np,np),TINY
      PARAMETER (NMAX= 4,TINY=1.0E-20)
C
C  Gegeben sei die matrix a(1:n,1:n) mit dem physikalischen dimensionen
C  a(1:np,1:np). Dieses programm zerlegt die matrix nach dem verfahren
C  der LU-decomposition mit einer zeilenweise permutation auf sich
C  selbst. a und n werden vorgegeben. a wird nach gleichung 2.3.14
C  ausgegeben; indx(1:n) ist ein outputvektor, in dem die zeilen-
C  permutationen durch das partielle pivotisieren festgahalten werden;
C  d ist +1 oder -1, je nach dem, ob die zahl der permutationen grade
C  oder ungrade war. Dieses programm wird zusammen mit dem programm
C  lubksb zur loesung linearer gleichungen und zur invertierung von
C  matrizen benutzt.
C  Lit.: Mumerical Recipes
C
      INTEGER *4 i,imax,j,k
C                            vv speichert die implicite skalierung
C                            jeder zeile.
C
      REAL    *4 aamax,dum,sum,vv(NMAX)
C
C                            Bisher wurden keine zeilen vertauscht.
      d=1.
C                            Schleife ueber die zeilen, um implicite
C                            skalierungsinformation zu bekommen.
      do  i=1,n
        aamax=0.
        do  j=1,n
          if (abs(a(i,j)) .gt. aamax) aamax=abs(a(i,j))
        enddo
        if (aamax .eq. 0.) then
c         write (6,'(a)') 'ludcmp: matrix singulaer'
          d=0.
          return
        endif
C                            Skalierung speichern
        vv(i)=1./aamax
      enddo
C                            Schleife ueber die spalten nach Crout's
C                            methode (gl. 2.3.12 ausser fuer i=j).
      do  j=1,n
        do  i=1,j-1
          sum=a(i,j)
          do  k=1,i-1
            sum=sum-a(i,k)*a(k,j)
          enddo
          a(i,j)=sum
        enddo
C                            Initialisiere fuer das groesste
C                            pivotelement.
        aamax=0.
        do  i=j,n
          sum=a(i,j)
          do  k=1,j-1
            sum=sum-a(i,k)*a(k,j)
          enddo
          a(i,j)=sum
          dum=vv(i)*abs(sum)  ! Figure of merit fuer das pivot.
          if (dum .ge. aamax) then  ! Bestes bisher?
            imax=i
            aamax=dum
          endif
        enddo
        if (j .ne. imax) then  ! Muessen zeilen vertauscht werden?
          do  k=1,n            ! Leider ja ...
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
          enddo
          d=-d                 ! ... und paritaet von d aendern.
          vv(imax)=vv(j)       ! Tausche auch skalenfaktor.
        endif
        indx(j)=imax
        if (a(j,j) .eq. 0.) a(j,j)=TINY
C
C  Ist das pivot-element 0, ist die matrix singulaer (zumindest
C  bezogen auf die genauigkeit des algorithmus). Bei einigen
C  anwendungen auf singulaere matrizen ist es wuenschenswert, TINY
C  fuer 0 zu setzen.
C
        if (j .ne. n) then   ! Jetzt wird schliesslich durch das
          dum=1./a(j,j)      ! pivot-element dividiert.
          do  i=j+1,n
            a(i,j)=a(i,j)*dum
          enddo
        endif
      enddo                  ! Naechste zeilenreduktion
      return
      END
