C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CKPEHND   51             -------------------
                          SUBROUTINE ZVEHND
C CY-ENG-                -------------------
C
C              Programm zur Verarbeitung von Ebenen.
C
C     Kommando  NTYP  Bedeutung
C     EP          1   Ebene durch drei Punkte
C     EML         2   Ebene durch millersche Indices und d
C     EMP         3   Ebene durch mill. Ind. und einen Punkt
C     PEEE        4   Punkt durch drei Ebenen
C     LEP         5   Abstand eines Punktes von der Ebene
C     PVE         6   Punkt durch Vektor und Ebene
C     GESP        7   Generiere Ebenenschnittpunkte
C     EL          8   Ebene laden
C     ESP         9   Ebene speichern
C     WEE        10   Winkel zwischen Ebenen
C     FACE       11   = EML, d wird mit 10 multipliziert
C     SKE        12   Speichere Koordinatensystem als Ebene
C     CVNT       13   Konvertiere Normalenvektor nach triklin
C     CVNK       14   Konvertiere Normalenvektor nach kartesisch
C     ENP        15   Ebene durch Normalenvektor und Punkt
C
C                 Geschrieben: 311145 OCT 1983 (Hundt)
C                 Geaendert:   151017 APR 2008 (Hundt)
C
C                        -------------------
C
      IMPLICIT NONE
C
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
C
      INTEGER *4 IAIN(72),IBLANK,IUNDEF,NREBNE,I,J,K,IRET,L
      INTEGER *4 IPKTST,IE1,IE2,NTYPS
C
      REAL    *4 CDS(7),EBNAM(2),SUM,TEMP,TOL,ARCCOS
C
      EQUIVALENCE (AIN(1),IAIN(1)),(ICDS(1),CDS(1))
C
      DATA IBLANK/4H    /,IUNDEF/4HUNDF/,EBNAM/4HEBNE,4HSCHN/
C
      NTYPS=NTYP
      GO TO (10,50,70,90,140,160,170,240,250,270,50,250,290,290,320),
C            1  2  3  4  5   6   7   8   9   10  11 12  13  14  15
     + NTYP
C
C                            'EP' - Ebene durch drei Punkte
   10 NTRY=26
      ICDS(4)=NAT(NS)+1
      ICDS(5)=IUNDEF
      ICDS(6)=IBLANK
CYENG1
      CALL FLES (ICDS,ISKP(31),5H4CAAE,23Hc1,c2,c3,Nr,Name,Unt.?$)
CYENG2
CYENGY      CALL FLES (ICDS,ISKP(31),5H4CAAE,23Hc1,c2,c3,No,name,ext.?$)
      IF (ISKP(31) .GE. 3) GO TO 30
   20 IERR=51 0058 16
      GO TO 999
C
   30 NREBNE=ICDS(4)
      IF (NREBNE .LE. 0 .OR. NREBNE .GT. NATDM) GO TO 20
C
C                            Koordinaten
   40 NTYP=7
      DO  NC=1,3
        IF (ICDS(NC) .EQ. 0) THEN
          ICDS(NC)=NC
          CALL ZVGC01 (ICDS(NC))
        ENDIF
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
        DO  I=1,3
          X(I,NC)=X1(I)
        ENDDO
      ENDDO
C                            Vektoren
      DO  I=1,3
        X(I,2)=X(I,2)-X(I,1)
        X(I,3)=X(I,3)-X(I,1)
      ENDDO
C                            Kreuzprodukt und Normierung
      SUM=0.
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        TEMP=X(J,2)*X(K,3)-X(K,2)*X(J,3)
        X2(I)=TEMP
        SUM=SUM+TEMP*TEMP
      ENDDO
C                            Teste Gueltigkeit
      IF (SUM .LE. 0.) THEN
        IERR=51 0093 06
        GO TO 999
      ENDIF
C                            Bestimme d
      TEMP=0.
      SUM=SQRT(SUM)
      DO  I=1,3
        X2(I)=X2(I)/SUM
        TEMP=TEMP+X2(I)*X(I,1)
      ENDDO
      IF (TEMP .LT. 0.) THEN
        TEMP=-TEMP
        DO  I=1,3
          X2(I)=-X2(I)
        ENDDO
      ENDIF
C                            Speichere
      DO  I=1,3
        AT(I,NREBNE,NS)=X2(I)
      ENDDO
      AT(4,NREBNE,NS)=TEMP
      IF (ICDS(5) .NE. IUNDEF) THEN
        AT(6,NREBNE,NS)=CDS(6)
        AT(5,NREBNE,NS)=CDS(5)
        ICLRP(NREBNE,NS)=0
        IPFLG(NREBNE,NS)=0
      ENDIF
      AT(7,NREBNE,NS)=AT(5,NREBNE,NS)
      CALL CNVTUP (AT(7,NREBNE,NS),1,4)
      IF (NREBNE .gt. NAT(NS)) NAT(NS)=NREBNE
      GO TO 999
C
C                            Ebene durch millersche Indices und d
   50 ICDS(5)=NAT(NS)+1
      ICDS(6)=IUNDEF
      ICDS(7)=IBLANK
CYENG1
      CALL FLES (ICDS,ISKP(31),6H4FIAAE,22Hh,k,l,d,Nr,Name,Unt.?$)
CYENG2
CYENGY      CALL FLES (ICDS,ISKP(31),6H4FIAAE,22Hh,k,l,d,Nr,name,ext.?$)
      IF (ISKP(31) .LT. 4) GO TO 20
   60 NTRY=2
      NREBNE=ICDS(5)
      IAIN(4)=ICDS(4)
      IF (NTYPS .EQ. 11) AIN(4)=AIN(4)*CONST(42)
      DO  I=5,6
        ICDS(I)=ICDS(I+1)
      ENDDO
      IF (NREBNE .LE. 0 .OR. NREBNE .GT. NATDM) GO TO 20
C
      SUM=0.
      DO  I=1,3
        TEMP=RG(I,1,NS)*CDS(1)+RG(I,2,NS)*CDS(2)+RG(I,3,NS)*CDS(3)
        SUM=SUM+TEMP*TEMP
        X1(I)=TEMP
      ENDDO
      IF (SUM .LE. 0.) GO TO 20
      TEMP=SQRT(SUM)
      DO  I=1,3
        AT(I,NREBNE,NS)=X1(I)/TEMP
      ENDDO
      AT(6,NREBNE,NS)=CDS(6)
      AT(5,NREBNE,NS)=CDS(5)
      AT(7,NREBNE,NS)=CDS(5)
      CALL CNVTUP (AT(7,NREBNE,NS),1,4)
      ICLRP(NREBNE,NS)=0
      IPFLG(NREBNE,NS)=0
      IF (NREBNE .GT. NAT(NS)) NAT(NS)=NREBNE
      IF (NTYPS .EQ. 3) GO TO 80
      AT(4,NREBNE,NS)=AIN(4)
      GO TO 999
C                            'EMP' - Ebene durch h,k,l und
C                            und einen weiteren Punkt
   70 ICDS(5)=NAT(NS)+1
      ICDS(6)=IUNDEF
      ICDS(7)=IBLANK
CYENG1
      CALL FLES (ICDS,ISKP(31),7H3F2CAAE,22Hh,k,l,c,Nr,Name,Unt.?$)
CYENG2
CYENGY      CALL FLES (ICDS,ISKP(31),7H3F2CAAE,22Hh,k,l,c,Nr,name,ext.?$)
      IF (ISKP(32) .LE. 0) THEN
        IERR=51 0174 16
        GO TO 999
      ENDIF
      GO TO 60
C
   80 ICDS(4)=IAIN(4)
      IF (ICDS(4) .EQ. 0) THEN
        NTYP=7
        ICDS(4)=1
        CALL ZVGC01 (ICDS(4))
      ENDIF
      NTRY=26
      NC=4
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
C
C                            Bestimme neues d
C
      TEMP=AT(1,NREBNE,NS)*X1(1)+AT(2,NREBNE,NS)*X1(2)+
     + AT(3,NREBNE,NS)*X1(3)
      AT(4,NREBNE,NS)=TEMP
      IF (TEMP .LT. 0.) THEN
        DO  I=1,4
          AT(I,NREBNE,NS)=-AT(I,NREBNE,NS)
        ENDDO
      ENDIF
      GO TO 999
C                            'PEEE' - Schnittpunkt dreier Ebenen
   90 IRET=0
      CALL FLES(IAIN,ISKP(31),3H3IE,10He1,e2,e3?$)
      IF (ISKP(31) .LT. 3) THEN
        IERR=51 0205 16
        GO TO 999
      ENDIF
C                            Baue Matrix auf
  100 CONTINUE
        DO  I=1,3
        K=IAIN(I)
        IF (K .LE. 0 .OR. K .GT. NATDM) THEN
          IERR=51 0213 04
          GO TO 999
        ENDIF
C
        DO  L=1,3
          J=3*L+I
          AIN(J)=AT(L,K,NS)
        ENDDO
        X1(I)=AT(4,K,NS)
      ENDDO
C                            Invertiere Matrix
C
      CALL LUDCMP (AIN(4),3,3,AIN(17),AIN(20))
      TEMP=ABS(AIN(4)*AIN(8)*AIN(12))
      IF (IRET .GT. 0) THEN
        IF(TEMP .LT. 1.E-5) GO TO 230
        GO TO 120
      ENDIF
      IF (TEMP .GT. 1.E-5) GO TO 120
      IERR=51 0232 15
      GO TO 999
C
  120 CALL LUBKSB (AIN(4),3,3,AIN(17),X1)
      IF (IRET .GT. 0) GO TO 200
  130 DO  I=1,3
        PTRIKL(I)=RG(1,I,NS)*X1(1)+RG(2,I,NS)*X1(2)+RG(3,I,NS)*X1(3)
      ENDDO
      NTYP=16
      CALL ZVGE01
      GO TO 999
C                            Kommando 'LEP' - Abstand einer Ebene von
C                            einem Punkt
C
  140 CALL FLES (ICDS,ISKP(31),3H2CE,5He,c?$)
      IF (ISKP(31) .LT. 2) THEN
        IERR=51 0248 16
        GO TO 999
      ENDIF
C
      I=ICDS(1)
      IF (I .LE. 0 .OR. I .GT. NATDM) THEN
        IERR=51 0254 04
        GO TO 999
      ENDIF
C
      NC=2
      NTRY=26
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
C
      CONST(5)=ABS(AT(1,I,NS)*X1(1)+AT(2,I,NS)*X1(2)+
     + AT(3,I,NS)*X1(3)-AT(4,I,NS))
      WRITE (WRLINE,150) CONST(5)
      CALL KPWRTE (NTOUT)
CYENG1
  150 FORMAT ('Abstand =',F10.5)
CYENG2
CYENGY  150 FORMAT ('Distance =',F7.2)
      GO TO 999
C                            PVE - Punkt durch Vektor und Ebene
  160 DO  I=1,3
        ICDS(I)=0
      enddo
      CALL FLES (ICDS,ISKP(31),3H3CE,9Hp1,p2,e?$)
      IF (ICDS(3) .LE. 0 .OR. ICDS(3) .GT. NATDM) THEN
        IERR=51 0278 16
        GO TO 999
      ENDIF
C
      NTYP=7
      NTRY=26
      DO  NC=1,2
        IF (ICDS(NC) .EQ. 0) THEN
          ICDS(NC)=NC
          CALL ZVGC01 (ICDS(NC))
        ENDIF
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
        DO  I=1,3
          X(I,NC)=X1(I)
        ENDDO
      ENDDO
C
      DO  I=1,3
        X2(I)=X(I,2)-X(I,1)
      ENDDO
      J=ICDS(3)
      T(1)=AT(1,J,NS)*X2(1)+AT(2,J,NS)*X2(2)+AT(3,J,NS)*X2(3)
      IF (T(1) .EQ. 0.) THEN
        IERR=51 0302 15
        GO TO 999
      ENDIF
C
      T(2)=(AT(4,J,NS)-AT(1,J,NS)*X(1,1)-
     + AT(2,J,NS)*X(2,1)-AT(3,J,NS)*X(3,1))/T(1)
      DO  I=1,3
        X1(I)=X(I,1)+T(2)*X2(I)
      ENDDO
      GO TO 130
C                       'GESP' - Generiere Ebenenschnittpunkte
  170 IPKTST=NAT(NS)+1
      ISKP(64)=IPKTST
      IF (ISKP(14) .EQ. 0) THEN
        IAIN(1)=2
        IAIN(2)=ISKP(53)
        IAIN(3)=NAT(NS)
        AIN(4)=0.001
        CALL FLES (IAIN,ISKP(31),4H3CFE,15Hp, e1,e2, tol?$)
      ENDIF
      ICDS(1)=IAIN(1)
      NC=1
      NTRY=26
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
      DO  I=1,3
        T(I)=X1(I)
      ENDDO
      IE1=IAIN(2)
      IE2=IAIN(3)
      TOL=AIN(4)
      DO  J=IE1,IE2
        TEMP=AT(1,J,NS)*T(1)+AT(2,J,NS)*T(2)+AT(3,J,NS)*T(3)-AT(4,J,NS)
        IF (ABS(TEMP) .LT. TOL) THEN
          WRITE (WRLINE,'(A,I4,A)')
CYENG1
     +    'Der Punkt p darf auf keiner Ebene liegen (',j,').'
CYENG2
CYENGY     +    'The point p may not lay on one of the planes (',j,').'
          CALL KPWRTE (NTOUT)
          IERR=-1
          GO TO 999
        ENDIF
        AT(8,J,NS)=TEMP
      ENDDO
C
      IF (IE1 .LE. 0 .OR. IE2 .GT. NATDM .OR. IE1+2 .GT. IE2) THEN
        IERR=51 0347 16
        GO TO 999
      ENDIF
C
      IRET=1
      IAIN(1)=IE1
  180 IAIN(2)=IAIN(1)+1
  190 IAIN(3)=IAIN(2)+1
C                       Berechne Schnittpunkt
      GO TO 100
C                       Teste, ob der Schnittpunkt bei allen
C                       Ebenen auf der gleichen Seite liegt,
C                       wie der vorgegebene Punkt.
  200 CONTINUE
      DO  J=IE1,IE2
        IF (J .EQ. IAIN(1) .OR. J .EQ. IAIN(2) .OR. J .EQ. IAIN(3))
     +  GO TO 210
        AIN(4)=AT(1,J,NS)*X1(1)+AT(2,J,NS)*X1(2)+
     +  AT(3,J,NS)*X1(3)-AT(4,J,NS)
        IF (ABS(AIN(4)) .LE. TOL) GO TO 210
        IF (AIN(4)*at(8,j,ns) .LT. 0.) GO TO 230
  210   CONTINUE
      ENDDO
C                       Merke die beteiligten Ebenen.
C
      ICLRP(IAIN(1),NS)=ICLRP(IAIN(1),NS)+1
      ICLRP(IAIN(2),NS)=ICLRP(IAIN(2),NS)+1
      ICLRP(IAIN(3),NS)=ICLRP(IAIN(3),NS)+1
C
C                       Nimm Punkt auf...
      DO  I=1,3
        T(I)=RG(1,I,NS)*X1(1)+RG(2,I,NS)*X1(2)+RG(3,I,NS)*X1(3)
      ENDDO
C
C                       ... sofern nicht schon vorhanden.
C
      IF (IPKTST .LE. NAT(NS)) THEN
        DO  J=IPKTST,NAT(NS)
          DO  I=1,3
            IF (ABS(T(I)-AT(I,J,NS)) .GT. CONST(21)) GO TO 220
          ENDDO
          GO TO 230
  220     CONTINUE
        ENDDO
      ENDIF
C
      IF (NAT(NS) .GE. NATDM) THEN
        IERR=51 0388 01
        GO TO 999
      ENDIF
C
      NAT(NS)=NAT(NS)+1
      DO  I=1,3
        AT(I,NAT(NS),NS)=T(I)
      ENDDO
      AT(4,NAT(NS),NS)=0.3
      AT(5,NAT(NS),NS)=EBNAM(1)
      AT(6,NAT(NS),NS)=EBNAM(2)
      AT(7,NAT(NS),NS)=EBNAM(1)
      ICLRP(NAT(NS),NS)=0
      IPFLG(NAT(NS),NS)=0
C
  230 IAIN(3)=IAIN(3)+1
      IF (IAIN(3) .LE. IE2) GO TO 100
      IAIN(2)=IAIN(2)+1
      IF (IAIN(2) .LT. IE2) GO TO 190
      IAIN(1)=IAIN(1)+1
      IF (IAIN(1)+2 .LE. IE2) GO TO 180
      GO TO 999
C
C                       Kommando 'EL'
  240 IAIN(1)=0
CYENG1
      CALL FLES (IAIN,ISKP(31),2HIE,5HNr.?$)
CYENG2
CYENGY      CALL FLES (IAIN,ISKP(31),2HIE,5HNo.?$)
      I=IAIN(1)
      IF (I .LE. 0 .OR. I .GT. NATDM) THEN
        IERR=51 0420 16
        GO TO 999
      ENDIF
C
      DO  J=1,3
        PTRIKL(J)=RG(1,J,NS)*AT(1,I,NS)*AT(4,I,NS)+
     +  RG(2,J,NS)*AT(2,I,NS)*AT(4,I,NS)+
     +  RG(3,J,NS)*AT(3,I,NS)*AT(4,I,NS)
      ENDDO
      NTYP=16
      CALL ZVGE01
      GO TO 999
C                       Kommando 'ESP' - Ebene speichern
  250 IAIN(1)=NAT(NS)+1
      IAIN(2)=0
      IAIN(3)=IBLANK
CYENG1
      CALL FLES (IAIN,ISKP(31),4HJAAE,15HNr.,Name,Unt.?$)
CYENG2
CYENGY      CALL FLES (IAIN,ISKP(31),4HJAAE,15HNo.,name,ext.?$)
      I=IAIN(1)
      IF (I .LE. 0 .OR. I .GT. NATDM) THEN
        IERR=51 0442 16
        GO TO 999
      ENDIF
C
      IF (NTYP .EQ. 12) THEN
C                               'SKE' - Speichere Koord.-S.  als Ebene
        DO  J=1,3
          X(J,1)=0.
          AT(J,I,NS)=DREH(3,J)
        ENDDO
        NC=1
        NTRY=4
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
        AT(4,I,NS)=-X2(3)
        IF (AT(4,I,NS) .GE. 0.) GO TO 260
        DO  J=1,4
          AT(J,I,NS)=-AT(J,I,NS)
        ENDDO
        GO TO 260
      ENDIF
C
      TEMP=SQRT(PFREI(1)*PFREI(1)+PFREI(2)*PFREI(2)+PFREI(3)*PFREI(3))
      IF (TEMP .LE. 0.) THEN
        IERR=51 0466 06
        GO TO 999
      ENDIF
C
      AT(4,I,NS)=TEMP
      ICLRP(I,NS)=0
      IPFLG(I,NS)=0
      SUM=0.
      DO  J=1,3
        TEMP=DG(J,1,NS)*PTRIKL(1)+DG(J,2,NS)*PTRIKL(2)+
     +  DG(J,3,NS)*PTRIKL(3)
        AT(J,I,NS)=TEMP
        SUM=SUM+TEMP*TEMP
      ENDDO
      IF (SUM .LE. 0.) THEN
        IERR=51 0481 06
        GO TO 999
      ENDIF
      SUM=SQRT(SUM)
      DO  J=1,3
        AT(J,I,NS)=AT(J,I,NS)/SUM
      ENDDO
  260 IF (I .GT. NAT(NS)) NAT(NS)=I
      IF (IAIN(2) .EQ. 0) GO TO 999
      AT(5,I,NS)=AIN(2)
      AT(6,I,NS)=AIN(3)
      AT(7,I,NS)=AIN(2)
      CALL CNVTUP (AT(7,I,NS),1,4)
      GO TO 999
C                            'WEE' - Winkel zwischen Ebenen
C
  270 CALL FLES (IAIN,ISKP(31),3H2IE,9Hne1,ne2?$)
      DO  I=1,2
        IF (IAIN(I) .LE. 0) GO TO 330
        IF (IAIN(I) .GT. NAT(NS)) GO TO 330
      ENDDO
      I=IAIN(1)
      J=IAIN(2)
      TEMP=ARCCOS(AT(1,I,NS)*AT(1,J,NS)+
     + AT(2,I,NS)*AT(2,J,NS)+AT(3,I,NS)*AT(3,J,NS))
      CONST(6)=TEMP
      WRITE (WRLINE,280) TEMP
      CALL KPWRTE (NTOUT)
CYENG1
  280 FORMAT ('Winkel:',F7.2)
CYENG2
CYENGY  280 FORMAT ('Angle:',F7.2)
      GO TO 999
C                             'CVNT' - Konv. normalenvektor n. triklin
  290 IAIN(1)=ISKP(53)
      IAIN(2)=0
      CALL FLES (IAIN,ISKP(31),3H2IE,9Hne1,ne2?$)
      DO  I=1,2
        IF (IAIN(I) .LE. 0 .OR. IAIN(I) .GT. NAT(NS)) GO TO 330
      ENDDO
      IF (NTYP .EQ. 14) GO TO 300
      DO  I=IAIN(1),IAIN(2)
        DO  J=1,3
          X1(J)=RG(1,J,NS)*AT(1,I,NS)+
     +    RG(2,J,NS)*AT(2,I,NS)+RG(3,J,NS)*AT(3,I,NS)
        ENDDO
        DO  J=1,3
          AT(J,I,NS)=X1(J)*AT(4,I,NS)
        ENDDO
        AT(4,I,NS)=0.3
      ENDDO
      GO TO 999
C                            ... CVNK
  300 DO  I=IAIN(1),IAIN(2)
        TEMP=0.
        DO  J=1,3
          X1(J)=DG(J,1,NS)*AT(1,I,NS)+
     +    DG(J,2,NS)*AT(2,I,NS)+DG(J,3,NS)*AT(3,I,NS)
          TEMP=TEMP+X1(J)**2
        ENDDO
        TEMP=SQRT(TEMP)
        DO  J=1,3
          AT(J,I,NS)=X1(J)/TEMP
        ENDDO
        AT(4,I,NS)=TEMP
      ENDDO
      GO TO 999
C                            ENP - Ebene durch vektor und punkt
  320 CONTINUE
      IF (ISKP(14) .EQ. 0) THEN
        ICDS(1)=0
        ICDS(2)=0
        CDS(3)=0.
        ICDS(6)=NAT(NS)+1
CYENG1
        CALL FLES(ICDS,ISKP(31),7H2CFAAIE,23Hc1,c2,f,name,unt.,pos?$)
CYENG2
CYENGY        CALL FLES(ICDS,ISKP(31),7H2CFAAIE,23Hc1,c2,f,name,ext.,pos?$)
      ENDIF
      IF (ICDS(6) .LE. 0 .OR. ICDS(6) .GT. NATDM) THEN
        IERR=51 558 16
        GO TO 999
      ENDIF
      NTRY=26
      DO  NC=1,2
        IF (ICDS(NC) .EQ. 0) THEN
          ICDS(NC)=NC
          NTYP=7
          CALL ZVGC01(ICDS(NC))
        ENDIF
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
        DO  I=1,3
          X(I,NC)=X1(I)
        ENDDO
      ENDDO
C                            Vektor und schwerpunkt
      DO  I=1,3
        X2(I)=(X(I,1)+X(I,2))/2.
        X1(I)=X(I,2)-X(I,1)
      ENDDO
      SUM=SQRT(X1(1)**2+X1(2)**2+X1(3)**2)
      IF (SUM .EQ. 0.) THEN
        IERR=51 0581 06
        GO TO 999
      ENDIF
      DO  I=1,3
        X1(I)=X1(I)/SUM
        AT(I,ICDS(6),NS)=X1(I)
        X2(I)=X2(I)+(X(I,2)-X(I,1))*CDS(3)
      ENDDO
      SUM=X1(1)*X2(1)+X1(2)*X2(2)+X1(3)*X2(3)
      IF (SUM .LT. 0.) THEN
        SUM=-SUM
        DO  I=1,3
          AT(I,ICDS(6),NS)=-AT(I,ICDS(6),NS)
        ENDDO
      ENDIF
      AT(4,ICDS(6),NS)=SUM
      ICLRP(ICDS(6),NS)=0
      IF (ICDS(6) .GT. NAT(NS)) NAT(NS)=ICDS(6)
      IF (ISKP(33) .GT. 0) THEN
        AT(5,ICDS(6),NS)=CDS(4)
        AT(7,ICDS(6),NS)=CDS(4)
        CALL CNVTUP(CDS(7),1,4)
        AT(6,ICDS(6),NS)=CDS(5)
      ENDIF
      GO TO 999
C
  330 IERR=51 0606 16
C
  999 RETURN
      END
