C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CKPGNKO   52             -------------------
                          SUBROUTINE ZVGNKO
C  CY-ENG-               -------------------
C
C  Das Programm soll ein Koordinatensystem fuer den Plot generieren.
C  Hierzu werden ausgehend vom Punkt auf dem Punktregister 3 Punkte auf
C  der a-, b- und c-Achse berechnet und zusammen mit dem Punkt auf dem
C  Punktregister in die Parameterliste geladen.
C  Exsitiert bereits in der Parameterliste Atome mit den Namen "0", a,
C  b und c (unmittelbar hintereinander), so wird angenommen, dass
C  bereits ein Koordinatensystem angelegt wurde. Diese Punkte werden
C  dann fuer eine Neudefinition benutzt.
C
C  NTYP=1  GNKO Generiere Koordinatenkreuz
C  NTYP=2  GRZ  Generiere reziproke Zelle
C  NTYP=3  IOMX Importiere Orientierungsmatrix
C
C                geschrieben:  130800 MAR 1984  (Hundt)
C                geaendert:    150931 JUN 2004  (Hundt)
C
C                        -------------------
      IMPLICIT NONE
C
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
      INCLUDE 'RDBFR.INC'
C
      INTEGER *4 I,J,K,IAIN(72),JTO,INC,IPKDO,IV,IPOS,IRD,ICODE
      REAL    *4 ANAME(4),ATKO(4),VQK(IPKDIM,2),BLANK
      REAL    *4 TEMP,TEMP1,TEMP2,TEMP3,TEMP4,TEMPL,OMAT(3,3)
      REAL    *4 FLAM1,FLAM2,BETA,ARCCOS
C
      EQUIVALENCE (IPK(1,1),VQK(1,1)),(AIN(1),IAIN(1))
C
      DATA BLANK/4H    /,ATKO/1H0,1Ha,1Hb,1Hc/
      DATA ANAME/4HA   ,4HB   ,4HC   ,4HSTRN/
C
      IF (NTYP .GT. 1) GO TO 80
C
C                            GNKO. Hole die Laengen der Achsen.
      DO  I=1,3
        AIN(I)=1.2/O9
      ENDDO
      CALL FLES (AIN,ISKP(31),3H3FE,10Hl1,l2,l3?$)
C
C                            Suche nach einem Atom "0".
      IF (NAT(NS) .LT. 4) GO TO 60
      DO  IPOS=1,NAT(NS)-3
        DO  J=0,3
          IF ((AT(5,IPOS+J,NS) .NE. ATKO(J+1)) .OR.
     +    (AT(6,IPOS+J,NS) .NE. BLANK)) GO TO 50
        ENDDO
C                            Gefunden. Delete
C                            Plotkommandos (sofern vorhanden).
        J=1
        IF (IPKP(NS) .GT. 0) THEN
   10     JTO=J
          INC=IPK(J,NS)
          IPKDO=IPK(J+1,NS)
          IF (IPKDO .EQ. 2) THEN
C
C                              Teste VB-Kommando.
C
            IF ((IPK(J+2,NS) .NE. IPOS) .OR.
     +          (IPK(J+3,NS) .NE. IPOS) .OR.
     +          (IPK(J+3,NS) .NE. IPOS+1) .OR.
     +          (IPK(J+3,NS) .NE. IPOS+3)) GO TO 20
            GO TO 40
          ELSE IF (IPKDO .EQ. 5) THEN
C                              Teste BA-Kommando.
            DO  I=1,3
              IF (IPK(3*I+J+2,NS) .NE. (IPOS+I)*200000+111001) GO TO 20
            ENDDO
            GO TO 40
          ENDIF
C                              Skip Plotkommando.
   20     J=J+INC
          IF (J .LT. IPKP(NS)) GO TO 10
          GO TO 70
C                                Delete Plotkommando.
   30     IPK(JTO,NS)=IPK(IV,NS)
          JTO=JTO+1
   40     IV=JTO+INC
          IF (IV .LE. IPKP(NS)) GO TO 30
          IPKP(NS)=JTO-1
          IF (J .LT. IPKP(NS)) GO TO 10
        ENDIF
        GO TO 70
   50 CONTINUE
      ENDDO
C                            Nicht gefunden. Erzeuge neue Atome.
C
   60 IF (NAT(NS)+4 .GT. NATDM) THEN
        IERR=52003102
        GO TO 999
      ENDIF
      DO  I=1,4
        NAT(NS)=NAT(NS)+1
        AT(4,NAT(NS),NS)=.1
        AT(5,NAT(NS),NS)=ATKO(I)
        AT(6,NAT(NS),NS)=BLANK
        AT(7,NAT(NS),NS)=ATKO(I)
        CALL CNVTUP (AT(7,NAT(NS),NS),1,4)
        ICLRP(NAT(NS),NS)=0
        IPFLG(NAT(NS),NS)=0
      ENDDO
      IPOS=NAT(NS)-3
C                            Generiere und Koordinaten ...
   70 DO  I=1,3
        DO  J=0,3
          AT(I,IPOS+J,NS)=PTRIKL(I)
        ENDDO
        AT(I,IPOS+I,NS)=PTRIKL(I)+AIN(I)/ZELLE(I,NS)
      ENDDO
C                            ... und Codes.
      ICDS(1)=IPOS
      ICDS(2)=IPOS+3
      ISKP(31)=2
      NTYP=1
      ISKP(14)=1
      CALL ZVASCI
      ISKP(14)=0
      IF (IERR .NE. 0) GO TO 999
C
C                            Beschriftungskommando
C
      IF (IPKP(NS)+27 .GT. IPKDM) THEN
        IERR=52029011
        GO TO 999
      ENDIF
C                            Erzeuge Header
      IPK(IPKP(NS)+1,NS)=14
      IPK(IPKP(NS)+2,NS)=5
      IPK(IPKP(NS)+3,NS)=ISKP(41)
      IPK(IPKP(NS)+4,NS)=ISKP(49)
      VQK(IPKP(NS)+5,NS)=CONST(15)
C                            Freie Koordinaten
      NTRY=4
      DO  NC=1,4
        DO  I=1,3
          X(I,NC)=AT(I,IPOS+NC-1,NS)
        ENDDO
        CALL ZVDEK
        DO  I=1,3
          X(I,NC)=X2(I)
        ENDDO
      ENDDO
C                            Vektoren in der Zeichenebene
      TEMPL=0.
      DO  I=1,3
        TEMP1=X(1,I+1)-X(1,1)
        TEMP2=X(2,I+1)-X(2,1)
        TEMP3=X(3,I+1)-X(3,1)
        TEMP4=TEMP1*TEMP1+TEMP2*TEMP2
        TEMPL=AMAX1(TEMPL,TEMP3*TEMP3+TEMP4)
C
        IF (TEMP4 .LE. 0.) THEN
          TEMP1=1.5*CONST(15)
          TEMP2=-0.5*CONST(15)
        ELSE
          TEMP4=SQRT(TEMP4)
          TEMP1=(1.5*TEMP1/TEMP4-0.5)*CONST(15)
          TEMP2=(1.5*TEMP2/TEMP4-0.5)*CONST(15)
        ENDIF
C
C                              Rest des Beschrifungskommandos
C
        IPK(I*3+IPKP(NS)+3,NS)=(IPOS+I)*200000+111001
        VQK(I*3+IPKP(NS)+4,NS)=TEMP1
        VQK(I*3+IPKP(NS)+5,NS)=TEMP2
      ENDDO
      IPKP(NS)=IPKP(NS)+14
C                            VB-Kommando
      IPK(IPKP(NS)+1,NS)=13
      IPK(IPKP(NS)+2,NS)=2
      IPK(IPKP(NS)+3,NS)=IPOS
      IPK(IPKP(NS)+4,NS)=IPOS
      IPK(IPKP(NS)+5,NS)=IPOS+1
      IPK(IPKP(NS)+6,NS)=IPOS+3
      VQK(IPKP(NS)+7,NS)=.01
      VQK(IPKP(NS)+8,NS)=SQRT(TEMPL)+.1
      VQK(IPKP(NS)+9,NS)=.01
      IPK(IPKP(NS)+10,NS)=11
      IPK(IPKP(NS)+11,NS)=0
      IPK(IPKP(NS)+12,NS)=0
      IPK(IPKP(NS)+13,NS)=0
      IPKP(NS)=IPKP(NS)+13
      GO TO 999
C
C                            GRZ. Generiere reziproke Gitterpunkte
C
   80 IF (NAT(NS)+3 .GT. NATDM) THEN
        IERR=52038002
        GO TO 999
      ENDIF
C
      AIN(1)=0.3
      IF (NTYP .EQ. 3) AIN(1)=0.003
      IAIN(2)=0
CYENG1
      CALL FLES (AIN,ISKP(31),3HFIE,8Hr,frbz?$)
CYENG2
CYENGY      CALL FLES (AIN,ISKP(31),3HFIE,8Hr,clrp?$)
      IF (NTYP .GT. 2) GO TO 100
      DO  I=1,3
        DO  J=1,3
          AT(J,NAT(NS)+I,NS)=RG(1,J,NS)*RG(1,I,NS)+
     +    RG(2,J,NS)*RG(2,I,NS)+RG(3,J,NS)*RG(3,I,NS)
        ENDDO
        AT(4,NAT(NS)+I,NS)=AIN(1)
        AT(5,NAT(NS)+I,NS)=ANAME(I)
        AT(6,NAT(NS)+I,NS)=ANAME(4)
        AT(7,NAT(NS)+I,NS)=ANAME(I)
        CALL CNVTUP (AT(7,NAT(NS)+I,NS),1,4)
        ICLRP(NAT(NS)+I,NS)=IAIN(2)
        IPFLG(NAT(NS)+I,NS)=0
      ENDDO
      NAT(NS)=NAT(NS)+3
      GO TO 999
C                            IOMX - Importiere Orientierungsmatrix
  100 CONTINUE
      TEMP1=AIN(1)
      JTO=IAIN(2)
      NTYP=NTXR1
      CALL ZVEAEH
      IF (IERR .NE. 0) GO TO 999
      IF (NAT(NS) .LT. 2) NAT(NS)=2
      IF (NSYM(NS) .LT. 1) NSYM(NS)=1
  240 CALL KPRDLN (NTXR1,IRD)
      IF (IRD .NE. 0) GO TO 999
C
C                            Teste, ob R-Matrix
C
      IF (RDLINE .EQ. 'RMAT P') THEN
        DO  J=1,3
          CALL KPRDLN (NTXR1,IRD)
          READ (RDLINE,FMT='(3F12.8)') (S(J,K),K=1,3)
        ENDDO
C                            T-Matrix
        CALL KPRDLN (NTXR1,IRD)
        DO  J=1,3
          CALL KPRDLN (NTXR1,IRD)
          READ (RDLINE,FMT='(3F12.8)') (X(J,K),K=1,3)
        ENDDO
        CALL MINV (X,3,TEMP,IAIN(3),IAIN(6))
        DO  J=1,3
          DO  K=1,3
            OMAT(J,K)=S(J,1)*X(1,K)+S(J,2)*X(2,K)+S(J,3)*X(3,K)
          ENDDO
        ENDDO
        GO TO 250
      ENDIF
      CALL KPRDLN (NTXR1,IRD)
      IF (IRD .NE. 0) GO TO 999
      READ (RDLINE,'(2X,I2)',ERR=999) ICODE
      IF (ICODE .NE. 31) GO TO 240
c
c                          R-Matrix 1. Teil
c
      READ (RDLINE,FMT='(6X,3F9.6,2X,3F9.6)')
     1 OMAT(1,1),OMAT(1,2),OMAT(1,3),
     2 OMAT(2,1),OMAT(2,2),OMAT(2,3)
      CALL KPRDLN (NTXR1,IRD)
      IF (IRD .NE. 0) GO TO 999
      READ (RDLINE,'(2X,I2)',ERR=999) ICODE
      IF (ICODE .NE. 32) GO TO 999
c
c                            R-Matrix 2. Teil
c
      READ (RDLINE,FMT='(6X,3F9.6,2X,3F9.6)')
     1OMAT(3,1),OMAT(3,2),OMAT(3,3),FLAM1,FLAM2,BETA
  250 IF (ISKP(58) .GT. 0) THEN
        WRITE (WRLINE,'(A)') 'R-Matrix'
        CALL KPWRTE (NTOUT)
        DO  K=1,3
          WRITE (WRLINE,'(3F10.5))') (OMAT(K,J),J=1,3)
          CALL KPWRTE (NTDISP)
        ENDDO
        WRITE (WRLINE,'(A,3F10.5)')
     1  ' lamda1, lamda2, beta:',FLAM1,FLAM2,BETA
        CALL KPWRTE (NTDISP)
      ENDIF
C                            close NTXR1
      NTYP=NTXR1+100
      CALL ZVEAEH
C                            Erzeuge Atome
      DO  J=1,3
        NAT(NS)=NAT(NS)+1
        DO  K=1,3
          AT(K,NAT(NS),NS)=TRAINV(K,1,NS)*OMAT(1,J)+
     +    TRAINV(K,2,NS)*OMAT(2,J)+TRAINV(K,3,NS)*OMAT(3,J)
        ENDDO
        AT(4,NAT(NS),NS)=TEMP1
        AT(5,NAT(NS),NS)=ANAME(J)
        AT(6,NAT(NS),NS)=ANAME(4)
        AT(7,NAT(NS),NS)=ANAME(J)
        CALL CNVTUP (AT(7,NAT(NS),NS),1,4)
        ICLRP(NAT(NS),NS)=JTO
        IPFLG(NAT(NS),NS)=0
      ENDDO
c                            Berechne hieraus die Gitterkonstanten.
      DO  K=1,3
        DO  J=1,3
          S(J,K)=OMAT(J,K)
        ENDDO
      ENDDO
      CALL MINV (S,3,TEMP,AIN(3),AIN(6))
      IF (TEMP .EQ. 0.) THEN
        WRITE (WRLINE,'(A)') 'DET=0.'
        CALL KPWRTE (NTDISP)
        GO TO 999
      ENDIF
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        AIN(I)=S(I,1)*S(I,1)+S(I,2)*S(I,2)+S(I,3)*S(I,3)
        AIN(I+3)=S(J,1)*S(K,1)+S(J,2)*S(K,2)+S(J,3)*S(K,3)
      ENDDO
      DO  I=1,3
        IF (AIN(I) .LE. 0.) THEN
          WRITE (WRLINE,'(A,I3,A)')
CYENG1
     1    'Eine Gitterkonstante <= 0!'
CYENG2
CYENGY     1    'One lattice constant <= 0!'
          CALL KPWRTE (NTDISP)
          GO TO 999
        ENDIF
        AIN(I)=SQRT(AIN(I))
      ENDDO
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        AIN(I+3)=ARCCOS(AIN(I+3)/(AIN(J)*AIN(K)))
      ENDDO
C
      WRITE (WRLINE,'(A,3F10.5,3F10.3)')
CYENG1
     1 'Gitterkonstanten:',(AIN(I),I=1,6)
CYENG2
CYENGY     1 'Lattice constants:',(AIN(I),I=1,6)
      CALL KPWRTE (NTDISP)
C
  999 RETURN
      END
