C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CZVGRTS   28             -------------------
                          SUBROUTINE ZVGRTS
C  CY-ENG-               -------------------
C
C   Das Programm testet, ob die eingegebenen Symmetrien eine Gruppe
C   bilden.
C
C   NTYP  Bedeutung
C    1    GRTS - Gruppentest
C    2    C - Centering
C    3    EDS - Entferne doppelt vorhandene symmetrien
C    4    - Teste auf doppelt vorhandene symmetrien
C
C                         Geschrieben: Hundt
C                  Geaendert: 221507 NOV 2002  (Hundt)
C
C                        -------------------
C
      IMPLICIT NONE
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
C
      INTEGER *4 IAIN(72),I,J,K,IOPT,IERRI,IDN,IDNFLG,L,LA,M,MA,KEYIN
      INTEGER *4 MB
      INTEGER *1 I1AIN(288)
      REAL    *4 AIDN(12),ATEMP(12),TOL,TEMP,TEMP2
C
      EQUIVALENCE (AIN(1),IAIN(1))
C
      DATA AIDN /0.,1.,4*0.,1.,4*0.,1./,TOL/3./
C
      IF (NTYP .EQ. 4) GO TO 280
      IF (NTYP .EQ. 3) GO TO 260
      IF (NTYP .EQ. 2) THEN
C                            C - Zentrierung (allgemein)
C
        IF (ISKP(14) .EQ. 0) THEN
          DO  I=1,3
            AIN(I)=0.
          ENDDO
          CALL FLES (IAIN,ISKP(31),3H3FE,7Hx,y,z?$)
        ENDIF
        DO  I=1,3
          IF (AIN(I) .LT. 0. .OR. AIN(I) .GE. 1.) THEN
            IERR=28 0045 16
            GO TO 999
          ENDIF
          IF (ABS(AIN(1))+ABS(AIN(2))+ABS(AIN(3)) .LT. 0.04) THEN
            IAIN(10)=0
            GO TO 999
          ENDIF
        ENDDO
        NHMSNR(1,NS)=ISKP(24)
        NHMSNR(2,NS)=ISKP(24)
        NHMSNR(3,NS)=0
c                           Erzeuge symmetrie.
        DO  K=1,12
          ATEMP(K)=AIDN(K)
        ENDDO
        J=1
        DO  K=1,3
          ATEMP(J)=AIN(K)
          J=J+4
        ENDDO
C                            Teste, ob nicht schon vorhanden.
        DO  I=1,NSYM(NS)
          DO  J=1,12
            IF (ABS(ATEMP(J)-SYM(J,I,NS)) .GT. 0.01) GO TO 10
          ENDDO
C                            Gefunden, wenn hier.
          IAIN(10)=0
          GO TO 999
   10     CONTINUE
        ENDDO
C                            Nimm auf, wenn noch moeglich.
C
        IF (NSYM(NS) .LT. NSYMDM) THEN
          NSYM(NS)=NSYM(NS)+1
          DO  K=1,12
            SYM(K,NSYM(NS),NS)=ATEMP(K)
          ENDDO
          IOPT=1
          GO TO 20
        ELSE
          IERR=28 0085 03
          GO TO 999
        ENDIF
      ENDIF
C                            GRTS - Gruppentest
C
      IF (ISKP(14) .NE. 0) THEN
        IOPT=ISKP(14)
        GO TO 20
      ENDIF
      IOPT=0
      CALL FLES (IOPT,ISKP(31),2HIE,10Hopt [0]? $)
   20 IF (NSYM(NS) .LE. 0) THEN
        IERR=28 0098 05
        GO TO 999
      ENDIF
C                            Buegele zuerst die symmetrien glatt
      DO  I=1,NSYM(NS)
        DO  J=1,12
          IF (J .EQ. 1 .OR. J .EQ. 5 .OR. J .EQ. 9) THEN
            TEMP2=144.
          ELSE
            TEMP2=12.
          ENDIF
          TEMP=SYM(J,I,NS)*TEMP2
          K=NINT(TEMP)
          IF (ABS(FLOAT(K)-TEMP) .LT. TOL) SYM(J,I,NS)=FLOAT(K)/TEMP2
        ENDDO
      ENDDO
C                            Suche Identitaet
      IERRI=0
      IDN=0
      DO  I=1,NSYM(NS)
        DO  J=1,12
          IF (SYM(J,I,NS) .NE. AIDN(J)) GO TO 30
        ENDDO
        IDN=I
        IF (NTYP .EQ. 2) GO TO 110
        GO TO 40
   30   CONTINUE
      ENDDO
CYENG1
      WRITE (WRLINE,'(A)') 'ZVGRTS0068 - Identitaet abwesend.'
CYENG2
CYENGY      WRITE (WRLINE,'(A)') 'ZVGRTS0068 - Missing identity.'
      CALL KPWRTE (NTDISP)
      IERRI=1
      GO TO 240
C
   40 I=1
      IF (IDN .EQ. 1) GO TO 90
CYENG1
      WRITE(WRLINE,'(A)') 'ZVGRTS0079 - Identitaet nicht vorn.'
CYENG2
CYENGY      WRITE(WRLINE,'(A)') 'ZVGRTS0079 - Identity not in position 1.'
      CALL KPWRTE (NTDISP)
      IERRI=1+IERRI
      GO TO 90
C                            Teste, ob Symmetrien doppelt sind.
   50 DO  K=1,12
        IF (SYM(K,I,NS) .NE. SYM(K,J,NS)) GO TO 80
      ENDDO
      WRITE(WRLINE,60) I,J
      CALL KPWRTE (NTDISP)
CYENG1
   60 FORMAT('ZVGRTS0090 - Symmetrien',I3,' und',I3,' identisch.')
CYENG2
CYENGY   60 FORMAT('ZVGRTS0090 - Symmetries',I3,' and',I3,' equal.')
      IERRI=1+IERRI
      GO TO 80
   70 J=I
   80 J=J+1
      IF (J-NSYM(NS)) 50,50,100
   90 I=0
  100 I=I+1
      IF (I-NSYM(NS)) 70,110,110
C
C                            Teste, ob Symmetrien aufeinander
C                            angewendet eine vorhandene ergeben.
  110 I=1
C                            Reset FLG
  120 IDNFLG=0
      J=1
C                            Loesche Puffer
  130 DO  K=1,12
        ATEMP(K)=0.
      ENDDO
C                            Wende  i-te auf j-te Symmetrie an.
      LA=-3
      DO  L=1,3
        LA=LA+4
        MA=-4
        DO  M=1,3
          MA=MA+4
          TEMP=SYM(LA+M,I,NS)
          DO  K=1,4
            MB=LA+K-1
            ATEMP(MB)=ATEMP(MB)+TEMP*SYM(MA+K,J,NS)
          ENDDO
        ENDDO
C                            Normiere.
C
        TEMP=ATEMP(LA)+SYM(LA,I,NS)
        GO TO 150
  140   TEMP=TEMP-1.
  150   IF(TEMP-.95) 170,170,140
  160   TEMP=TEMP+1.
  170   IF(TEMP+.08) 160,160,180
  180   ATEMP(LA)=TEMP
      ENDDO
C                            Suche in den Symmetrien.
      DO  L=1,NSYM(NS)
        DO  K=1,12
          IF (ABS(ATEMP(K)-SYM(K,L,NS)) .GT. .00001) GO TO 190
        ENDDO
C                              Gefunden. Teste ob Identitaet.
        IF(L .EQ. IDN) IDNFLG=1
        GO TO 210
  190   CONTINUE
      ENDDO
      IF (IOPT .NE. 0) THEN
        IF (NSYM(NS) .LT. NSYMDM) THEN
          NSYM(NS)=NSYM(NS)+1
          DO  K=1,12
            SYM(K,NSYM(NS),NS)=ATEMP(K)
          ENDDO
          GO TO 210
        ELSE
          IERR=28 0213 03
          GO TO 999
        ENDIF
      ENDIF
      WRITE(WRLINE,200) I,J
      CALL KPWRTE (NTDISP)
CYENG1
  200 FORMAT('ZVGRTS0159 - Symmetrie',I3,' * ',I3,' nicht vorhanden.')
CYENG2
CYENGY  200 FORMAT('ZVGRTS0159 - Symmetry',I3,' * ',I3,' not found.')
      IERRI=1+IERRI
      IF (IERRI .GT. 10) THEN
CYENG1
        WRITE (WRLINE,'(A)') 'ZVGRTS0164 - u.s.w.'
CYENG2
CYENGY        WRITE (WRLINE,'(A)') 'ZVGRTS0164 - etc.'
        CALL KPWRTE (NTDISP)
        GO TO 240
      ENDIF
      IF (KEYIN(0) .EQ. ISKP(20)) THEN
        IERR=-1
        GO TO 240
      ENDIF
  210 CONTINUE
      J=J+1
      IF (J .LE. NSYM(NS)) GO TO 130
C
C                            Teste, ob i-te Symmetrie invertierbar.
C
      IF(IDNFLG .EQ. 0 .AND. NTYP .NE. 2) THEN
        WRITE (WRLINE,220) I
CYENG1
  220   FORMAT('ZVGRTS0184 - Symmetrie',I3,' besitzt keine Inverse.')
CYENG2
CYENGY  220 FORMAT('ZVGRTS0184 - Symmetry',I3,' has no inverse.')
        CALL KPWRTE (NTDISP)
        IERRI=1+IERRI
      ENDIF
  230 CONTINUE
      I=I+1
      IF (I .LE. NSYM(NS)) GO TO 120
C                            Abgesang.
  240 IF (IERRI .NE. 0) THEN
        WRITE (WRLINE,250)
CYENG1
  250   FORMAT ('ZVGRTS0197 - Symmetrien fehlerhaft.')
CYENG2
CYENGY  250 FORMAT('ZVGRTS0197 - Error(s) in symmetries.')
        CALL KPWRTE (NTDISP)
        IERR=-1
      ENDIF
      GO TO 999
C                            EDS - Entferne doppelte symmetrien
  260 DO  I=1,NSYM(NS)
        I1AIN(I)=1
      ENDDO
      DO  I=1,NSYM(NS)-1
        IF (I1AIN(I) .NE. 0) THEN
          DO  J=I+1,NSYM(NS)
            IF (I1AIN(J) .NE. 0) THEN
              DO  K=1,12
                IF (ABS(SYM(K,I,NS)-SYM(K,J,NS)) .GT. 0.001) GO TO 270
              ENDDO
              I1AIN(J)=0
            ENDIF
  270       CONTINUE
          ENDDO
        ENDIF
      ENDDO
C                            Komprimiere.
      I=0
      DO  J=1,NSYM(NS)
        IF (I1AIN(J) .NE. 0) THEN
          I=I+1
          DO  K=1,12
            SYM(K,I,NS)=SYM(K,J,NS)
          ENDDO
        ENDIF
      ENDDO
      NSYM(NS)=I
      IF (ISKP(14) .EQ. 0) THEN
        CALL FLES (IAIN,ISKP(31),1HE,2H?$)
      ENDIF
      GO TO 999
C                            Teste bei, ob es zellverkleinernde
C                            symmetrien gibt.
  280 DO  I=1,10
        AIN(I)=0.
      ENDDO
      DO  I=1,9,4
        AIN(I)=1.
      ENDDO
      DO  I=1,NSYM(NS)
        DO  J=2,12
          IF (J .NE. 5 .AND. J .NE. 9) THEN
            IF (SYM(J,I,NS) .NE. AIDN(J)) GO TO 290
          ENDIF
        ENDDO
        K=0
        DO  J=1,9,4
          IF (SYM(J,I,NS) .NE. 0.) THEN
            K=K+1
            M=J
          ENDIF
        ENDDO
        IF (K .EQ. 1) THEN
          IF (ISKP(58) .GT. 0) THEN
            WRITE (WRLINE,'(A,3(F8.5,A))')
CYENG1
     +      'Zelle verkleinernde Symmetrie gefunden:',
CYENG2
CYENGY     +          'Cell reducing symmetry found:',
     +      SYM(1,I,NS),'+x',SYM(5,I,NS),'+y',SYM(9,I,NS),'+z'
            CALL KPWRTE (NTDISP)
          ENDIF
          IAIN(10)=1
          IF (SYM(M,I,NS) .LT. AIN(M)) AIN(M)=SYM(M,I,NS)
        ENDIF
  290   CONTINUE
      ENDDO
C
  999 RETURN
      END
