C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CKPPEZA   62              -------------------
                           SUBROUTINE ZVPEZA
C                         -------------------
C
C  Das Programm berechnet einen Punkt in einer Ebene, der von zwei
C  vorgegebenen Punkten vorgegebene Abstaende hat.
C
C                geschrieben: 070855 MAR 1989 (Hundt)
C                geaendert:   071042 FEB 2000 (Hundt)
C
C                        -------------------
C
      IMPLICIT NONE
C
      INCLUDE 'ZVCOM.INC'
C
      INTEGER *4 IAIN(72),I,IDSTFG,ISKP14,J
      REAL    *4 D1STR,D1STRQ,D2STR,D2STRQ,DC1C2,DC1C2Q,DSTR
      REAL    *4 DSTRQ,HSTR,HSTRQ,TEMP,TEMP2
C
      EQUIVALENCE (AIN(1),IAIN(1))
C
C                            Save Koordinatensystem.
      NTYP=19
      CALL ZVK
      ISKP14=ISKP(14)
      ISKP(14)=1
C                            Hole Input.
      AIN(3)=CONST(5)
      AIN(4)=CONST(5)
      DO  I=5,9
        IAIN(I)=0
      ENDDO
      CALL FLES (AIN,ISKP(31),7H2C2F5CE,
     1 33Hc1,c2,d1,d2,cur,cx1,cx2,cy1,cy2?$)
      IF (ISKP(33) .GT. 1 .AND.
     +  (ISKP(33) .NE. 3 .OR. ISKP(33) .NE. 5)) THEN
        IERR=62 0034 16
        GO TO 330
      ENDIF
C                            Definiere Koordinatensystem neu, ...
C
      IF (IAIN(5) .NE. 0) THEN
C                            ... falls 1. Code nicht 0.
        DO  I=1,5
          ICDS(I)=IAIN(I+4)
        ENDDO
        NTYP=1
        CALL ZVK
        IF (IERR .NE. 0) GO TO 330
      ENDIF
C
C                            Setze distance defaults.
      IDSTFG=0
      IF (AIN(3) .LT. 0) THEN
        IDSTFG=1
        AIN(3)=-AIN(3)
      ENDIF
      IF (AIN(4) .EQ. 0.) AIN(4)=AIN(3)
C
C                            Berechne freie Punkte c1 und c2.
      ICDS(6)=IAIN(1)
      ICDS(7)=IAIN(2)
      NTRY=28
      DO  NC=6,7
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 330
        DO  I=1,3
          X(I,NC)=X2(I)
        ENDDO
      ENDDO
C                            Bestimme Abstand c1 - c2.
      DC1C2Q=0.
      DO  I=1,3
        TEMP2=X(I,7)-X(I,6)
        X(I,1)=TEMP2
        DC1C2Q=DC1C2Q+TEMP2*TEMP2
      ENDDO
      DC1C2=SQRT(DC1C2Q)
C                            Teste, ob Loesung moeglich.
C
      IF (AIN(3)+AIN(4) .GE. DC1C2) GO TO 210
  190 IERR=62 0079 15
      GO TO 330
C                            Berechne Laengen in der x-y-Ebene.
C
  210 D1STRQ=AIN(3)*AIN(3)-X(3,6)*X(3,6)
      IF (D1STRQ .LE. 0.) GO TO 190
      D1STR=SQRT(D1STRQ)
      D2STRQ=AIN(4)*AIN(4)-X(3,7)*X(3,7)
      IF (D2STRQ .LE. 0.) GO TO 190
      D2STR=SQRT(D2STRQ)
      DSTRQ=X(1,1)*X(1,1)+X(2,1)*X(2,1)
      IF (DSTRQ .LE. 0.) GO TO 190
      DSTR=SQRT(DSTRQ)
      IF (D1STR+D2STR .LE. DSTR) GO TO 190
C
C                            Berechne Punkt auf dem Vektor.
C
      TEMP=(DSTRQ+D1STRQ-D2STRQ)/(2.*DSTR)
      TEMP2=TEMP/DSTR
      DO  I=1,2
        X(I,2)=X(I,6)+TEMP2*X(I,1)
      ENDDO
      HSTRQ=D1STRQ-TEMP*TEMP
      IF (HSTRQ .LE. 0.) GO TO 190
      HSTR=SQRT(HSTRQ)/DSTR
C                            Berechne die zwei moeglichen
C                            Loesungspunkte.
C
      X(1,3)=X(1,2)+HSTR*X(2,1)
      X(2,3)=X(2,2)-HSTR*X(1,1)
      X(1,4)=X(1,2)-HSTR*X(2,1)
      X(2,4)=X(2,2)+HSTR*X(1,1)
C
C                            Teste, welcher naeher an 0 liegt.
      J=3
      IF (X(1,3)*X(1,3)+X(2,3)*X(2,3) .GT.
     +   (X(1,4)*X(1,4)+X(2,4)*X(2,4))) J=4
      IF (IDSTFG .NE. 0) J=7-J
C
C                            Kopiere Punkt aufs Punktregister.
      DO  I=1,3
        PTRIKL(I)=TRAINV(I,1,NS)*X(1,J)+TRAINV(I,2,NS)*X(2,J)+UR(I,NS)
      ENDDO
C
C                            Restore Koordinatensystem.
  330 NTYP=20
      CALL ZVK
C                            Berechne freien Punkt.
      NTYP=16
      CALL ZVGE01
      ISKP(14)=ISKP14
C
      RETURN
      END
