C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CZVSPVV   49             -------------------
                          SUBROUTINE ZVSPVV
C  CY-ENG-               -------------------
C
C    Das Programm berechnet
C    1) den Schnittpunkt zweier Vektoren im Raum (LSQ)
C    2) die Millerschen Indices der x-y-Ebene
C       des freien Koordinatensystems.
C    3) die Millerschen Indices einer Ebene
C    4) wie 1), aber aufgrund vorhandener trikliner Koordinaten
C    5) wie 3), ausgabe aber als FACE zeile
C    6) CNML Ersetzt atomnamen durch Millersche indices
C
C                 geschrieben: 071221 JUL 1983 (Hundt)
C                 geaendert:   211631 FEB 2005 (Hundt)
C
C                        -------------------
C
      IMPLICIT NONE
C
      INTEGER *4 IAIN(72),I,J,JJJ,MINDX,JE,KA,KE,KPOI,IRC
      REAL    *4 A,B,C,F,AINDEX,XI,BLANK
C
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
C
      EQUIVALENCE (AIN(1),IAIN(1)),(BLANK,ISKP(24))
C
      GO TO (10,140,330,80,330,360), NTYP
C            1  2   3   4  5   6
C                            SPVV
C
   10 CALL FLES (ICDS,ISKP(31),3H4CE,14Hp1,p2, q1,q2?$)
      IF (ISKP(31) .LT. 4) THEN
        IERR=49 0035 16
        GO TO 999
      ENDIF
C                            Berechne trikl. Koordinaten
      NTRY=24
      NTYP=7
      DO  NC=1,4
        IF (ICDS(NC) .EQ. 0) THEN
          ICDS(NC)=NC
          CALL ZVGC01 (ICDS(NC))
        ENDIF
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
      ENDDO
   80 DO  I=1,5
        AIN(I)=0.
      ENDDO
C                            Baue Matrix auf.
      DO  I=1,3
        A=X(I,2)-X(I,1)
        X(I,5)=A
        B=X(I,3)-X(I,4)
        X(I,6)=B
        C=X(I,3)-X(I,1)
C
        AIN(1)=AIN(1)+A*A
        AIN(2)=AIN(2)+A*B
        AIN(3)=AIN(3)+B*B
        AIN(4)=AIN(4)+A*C
        AIN(5)=AIN(5)+B*C
      ENDDO
C                            Invertiere Matrix
C
      AIN(6)=AIN(1)*AIN(3)-AIN(2)*AIN(2)
      IF (AIN(6) .EQ. 0.) THEN
        IERR=49 0070 15
        GO TO 999
      ENDIF
C                            Loesungen
C
      A=(AIN(4)*AIN(3)-AIN(2)*AIN(5))/AIN(6)
      B=(AIN(1)*AIN(5)-AIN(2)*AIN(4))/AIN(6)
C
C                            Berechne Mittelpunkt und Abstand
      DO  I=1,3
        X(I,1)=X(I,1)+A*X(I,5)
        X(I,2)=X(I,3)-B*X(I,6)
        PTRIKL(I)=(X(I,1)+X(I,2))*.5
      ENDDO
      NTRY=2
      DO  NC=1,2
        CALL ZVDEK
        IF (IERR .NE. 0) GO TO 999
        IF (NC .EQ. 1) THEN
          DO  I=1,3
            X2(I)=X1(I)
          ENDDO
        ENDIF
      ENDDO
      A=SQRT((X2(1)-X1(1))**2+(X2(2)-X1(2))**2+(X2(3)-X1(3))**2)
CYENG1
      WRITE (WRLINE,'(A,F10.5)') 'Abstand:',A
CYENG2
CYENGY      WRITE (WRLINE,'(A,F10.5)') 'Distance:',A
      CALL KPWRTE (NTDISP)
      NTYP=16
      CALL ZVGE01
      GO TO 999
C                            Kommando MILL.
  140 JJJ=0
      AIN(1)=.05
      CALL FLES (AIN,ISKP(31),2HFE,5Htol?$)
C
C                            Berechne die Millerschen Indices.
      DO  I=1,3
        X2(I)=TRA(3,I,NS)
      ENDDO
      C=TRA(3,1,NS)*UR(1,NS)+TRA(3,2,NS)*UR(2,NS)+TRA(3,3,NS)*UR(3,NS)
C
  160 IF (C .LT. 0) THEN
        DO  I=1,3
          X2(I)=-X2(I)
        ENDDO
        C=-C
      ENDIF
C                            Versuche ganzzahlig zu machen.
      DO  MINDX=1,5
        AINDEX=MINDX
        DO  I=1,3
          IF (ABS(X2(I))-AIN(1) .GT. 0.) THEN
            F=AINDEX/ABS(X2(I))
            DO  J=1,3
              X1(J)=X2(J)*F
              XI=.5
              IF (X1(J) .LT. 0.) XI=-.5
              IF (ABS(AINT(X1(J)+XI)-X1(J)) .GT. AIN(1)) GO TO 280
            ENDDO
C                            (Fast) ganzzahlige Version gefunden.
            GO TO 310
C                            NEXT I
          ENDIF
  280     CONTINUE
        ENDDO
C                            NEXT MINDX
      ENDDO
C                            Kein ganzzahliges Ergebnis gefunden.
C                            Schreibe urspruengliche Loesung ab.
      DO  I=1,3
        X1(I)=X2(I)
        IT(I)=0
      ENDDO
C                            Ergebnis
  310 CONTINUE
      IF (NTYP .EQ. 6) GO TO 380
      IF (NTYP .EQ. 5) THEN
        DO  I=1,3
          IT(I)=NINT(X1(I))
        ENDDO
        C=C/CONST(42)
        WRITE (WRLINE,'(A,3I4,F10.6)') 'FACE',IT,C
        CALL KPWRTE (NTPCH)
        GO TO 355
      ENDIF
C
      WRITE (WRLINE,320) JJJ,X1,C
      CALL KPWRTE (NTOUT)
  320 FORMAT ('(',I3,') hkl:',3F7.2,'  d:',F10.5)
      IF (NTYP .EQ. 3) GO TO 355
      GO TO 999
C                            Kommando 'MLLE' - Millersche
C                            Indices einer Ebene
  330 IAIN(1)=NAT(NS)
      IAIN(2)=0
      AIN(3)=.05
CYENG1
      CALL FLES (AIN,ISKP(31),4H2IFE,13Hvon,bis,tol?$)
CYENG2
CYENGY      CALL FLES (AIN,ISKP(31),4H2IFE,13Hfrom,to,tol?$)
      J=IAIN(1)
      JE=IAIN(2)
      IF (JE .EQ. 0) JE=J
      IF (J .LE. 0) GO TO 340
      IF (JE .GT. NAT(NS)) GO TO 340
      IF (J .LE. JE) GO TO 350
  340 IERR=49 0179 19
      GO TO 999
C
  350 IF (NTYP .EQ. 5) THEN
        NTYP=NTPCH
        CALL ZVEAEH
        NTYP=5
      ENDIF
      AIN(1)=AIN(3)
      JJJ=J-1
  355 JJJ=JJJ+1
      IF (JJJ .GT. JE) GO TO 999
  356 C=AT(4,JJJ,NS)
      DO  I=1,3
        X2(I)=DG(1,I,NS)*AT(1,JJJ,NS)+DG(2,I,NS)*AT(2,JJJ,NS)+
     +  DG(3,I,NS)*AT(3,JJJ,NS)
      ENDDO
      GO TO 160
C                            CNML - Ersetze Nummer durch mill. Ind.
  360 CONTINUE
      AIN(3)=.05
CYENG1
      CALL FLES (AIN,ISKP(31),4H2IFE,13Hvon,bis,tol?$)
CYENG2
CYENGY      CALL FLES (AIN,ISKP(31),4H2IFE,13Hfrom,to,tol?$)
      KA=IAIN(1)
      KE=IAIN(2)
      AIN(1)=AIN(3)
      IF (KA .LT. ISKP(53) .OR. KE .GT. NAT(NS) .OR. KA .GT. KE) THEN
        IERR=49 0208 16
        GO TO 999
      ENDIF
  370 KPOI=0
      CALL FCNV (JJJ,ISKP(31),2HIE,AT(5,KA,NS),KPOI,8,IRC)
      IF (IRC .NE. 0 .OR. JJJ .LT. ISKP(53) .OR. JJJ .GT. NAT(NS)) THEN
        IERR=49 0214 16
        GO TO 999
      ENDIF
      GO TO 356
C
  380 DO  I=1,3
        IT(I)=NINT(X1(I))
      ENDDO
      AT(5,KA,NS)=BLANK
      AT(6,KA,NS)=BLANK
      KPOI=0
      DO  I=1,3
        CALL KPNTOS (IT(I),AT(5,KA,NS),KPOI,0)
      ENDDO
      KA=KA+1
      IF (KA .LE. KE) GO TO 370
C
  999 RETURN
      END
