C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
C         77              -------------------
                           SUBROUTINE ZVSYAN
C                         -------------------
C
C--- Das programm analysiert die symmetrie iain(1) und gibt ggf.
C    einzelheiten aus.
C
C                 geschrieben: 090850 feb 1997 (Hundt)
C                  bearbeitet: 011452 APR 2005 (Hundt)
C
C                         ------------------
C
      IMPLICIT NONE
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
C
      INTEGER *4 IAIN(72),J,J1,K,M,N,ISP,ITT,ITYPP(7),ITYPM(7)
      INTEGER *4 K1,K2,K3
      REAL    *4 DET,TOL,CDS(7)
C
      EQUIVALENCE (AIN(1),IAIN(1)),(ICDS(1),CDS(1))
C
      DATA TOL/0.001/
      DATA ITYPP/10,10,2,3,4,6,1/
      DATA ITYPM/-1,-6,-4,-3,-2,10,10/
C
      M=IAIN(1)
      DET=SYM(2,M,NS)* SYM(7,M,NS)*SYM(12,M,NS)+SYM(3,M,NS)* SYM(8,M,NS)
     + *SYM(10,M,NS)+SYM(4,M,NS)* SYM(6,M,NS)*SYM(11,M,NS)-SYM(4,M,NS)
     + * SYM(7,M,NS)*SYM(10,M,NS)-SYM(2,M,NS)* SYM(8,M,NS)*SYM(11,M,NS)
     + -SYM(3,M,NS)* SYM(6,M,NS)*SYM(12,M,NS)
      IF (ABS(DET)-1. .GT. TOL) THEN
        WRITE (WRLINE,'(A,F12.5,A)')
CYENG1
     +  'Determinante:',DET,' nicht erlaubt.'
CYENG2
CYENGY     +  'Determinant:',DET,' invalid.'
        GO TO 100
      ENDIF
      AIN(3)=SYM(2,M,NS)+SYM(7,M,NS)+SYM(12,M,NS)
      ISP=AIN(3)+4.5
      IF (ISP .LT. 0 .OR. ISP .GT. 7) THEN
        WRITE (WRLINE,'(A,I2,A,F12.5,A)')
CYENG1
     +  'Spur:',AIN(3),' nicht erlaubt.'
CYENG2
CYENGY     +  'Trace:',AIN(3),' invalid.'
        GO TO 100
      ENDIF
      IF (DET .GT. 0.) THEN
        ITT=ITYPP(ISP)
      ELSE
        ITT=ITYPM(ISP)
      ENDIF
C                            Triviale faelle: 1 und -1
C
      IF (ABS(ITT) .EQ. 1) THEN
        K1=0
        K2=0
        K3=0
        IF (ITT .EQ. 1) THEN
          J1=1
          DO  J=1,3
            X2(J)=SYM(J1,M,NS)
            T(J)=0.
            J1=J1+4
          ENDDO
          GO TO 90
        ELSE
          J1=1
          DO  J=1,3
            T(J)=SYM(J1,M,NS)*0.5
            X2(J)=0.
            J1=J1+4
          ENDDO
          GO TO 90
        ENDIF
      ENDIF
C                            Suche den vektor, der in sich selbst
C                            bzw. in sein spiegelbild ueberfuehrt wird.
      DO  K1=0,2
        X1(1)=K1
        DO  K2=2,-2,-1
          X1(2)=K2
          DO  K3=2,-2,-1
            IF (ABS(K1)+ABS(K2)+ABS(K3) .EQ. 0) GO TO 10
            X1(3)=K3
            K=1
            AIN(3)=0.
            DO  J=1,3
              X2(J)=SYM(K+1,M,NS)*X1(1)+SYM(K+2,M,NS)*X1(2)+
     +        SYM(K+3,M,NS)*X1(3)
              IF (ITT .LT. 0) THEN
                AIN(3)=AIN(3)+ABS(X2(J)+X1(J))
              ELSE
                AIN(3)=AIN(3)+ABS(X2(J)-X1(J))
              ENDIF
              IF (AIN(3) .GT. TOL) GO TO 10
              K=K+4
            ENDDO
            GO TO 20
   10       CONTINUE
          ENDDO
        ENDDO
      ENDDO
CYENG1
      WRITE (WRLINE,'(A)') 'Keine Achse gefunden.'
CYENG2
CYENGY      WRITE (WRLINE,'(A)') 'No axis found.'
      GO TO 100
C                            Achse gefunden. Fuehre KML aus.
   20 CONTINUE
      DO  J=1,3
        CDS(J)=X1(J)
        ICDS(J+3)=0
        UR(J,NS)=0.
      ENDDO
      NTYP=14
      ISKP(14)=1
      ISKP(32)=0
      CALL ZVK
      IF (IERR .NE. 0) GO TO 999
      IF ((ABS(SYM(1,M,NS))+ABS(SYM(5,M,NS))+ABS(SYM(9,M,NS)) .LE. 0.))
     +  THEN
        DO  J=1,3
          X2(J)=0.
          T(J)=0.
        ENDDO
        GO TO 90
      ENDIF
C
C                            Was wird aus dem nullpunkt, wenn er
C                            der symmetrie unterworfen wird?
      DO  J=1,3
        X1(J)=TRA(J,1,NS)*SYM(1,M,NS)+TRA(J,2,NS)*SYM(5,M,NS)+
     +  TRA(J,3,NS)*SYM(9,M,NS)
      ENDDO
C                            Unterscheidung der verschiedenen
C                            symmetrie-typen.
      IF (ITT .EQ. -2) THEN
C                            Liegt der bildpunkt auf der achse?
C
        IF (ABS(X1(1))+ABS(X1(2)) .LT. TOL) THEN
          K=1
          DO  J=1,3
            X2(J)=0.
            T(J)=SYM(K,M,NS)*0.5
            K=K+4
          ENDDO
          GO TO 90
        ELSE
C                            Berechne die triklinen Koordinaten
C                            des auf die achse projizieren punktes.
          K=1
          DO  J=1,3
            T(J)=TRAINV(J,3,NS)*X1(3)
            X2(J)=SYM(K,M,NS)-T(J)
            K=K+4
            T(J)=T(J)*0.5
          ENDDO
          GO TO 90
        ENDIF
      ENDIF
C                            Berechne fuer die normalen drehachsen
C                            den hub; setze ihn andernfalls=0.
      IF (ITT .GT. 0) THEN
        DO  J=1,3
          X2(J)=TRAINV(J,3,NS)*X1(3)
        ENDDO
      ELSE
        DO  J=1,3
          X2(J)=0.
        ENDDO
      ENDIF
C                            Erzeuge jetzt gemaesz der zaehligkeit
C                            punkte, um einen punkt auf dem
C                            symmetrieelement zu finden.
      K=ABS(ITT)
      IF (ITT .EQ. -3) K=6
      DO  J=1,3
        X(J,1)=0.
        T(J)=0.
      ENDDO
      DO  N=2,K
        J1=1
        DO  J=1,3
          X(J,N)=SYM(J1,M,NS)+SYM(J1+1,M,NS)*X(1,N-1)+
     +    SYM(J1+2,M,NS)*X(2,N-1)+SYM(J1+3,M,NS)*X(3,N-1)
          J1=J1+4
        ENDDO
        DO  J=1,3
          X1(J)=TRA(J,1,NS)*X(1,N)+TRA(J,2,NS)*X(2,N)+TRA(J,3,NS)*X(3,N)
        ENDDO
        IF (ITT .GT. 0) X1(3)=0.
        DO  J=1,3
          T(J)=T(J)+X1(J)
        ENDDO
      ENDDO
      AIN(4)=1./FLOAT(K)
      DO  J=1,3
        X1(J)=T(J)*AIN(4)
      ENDDO
      DO  J=1,3
        T(J)=TRAINV(J,1,NS)*X1(1)+TRAINV(J,2,NS)*X1(2)+
     +  TRAINV(J,3,NS)*X1(3)
      ENDDO
C
   90 CONTINUE
      IF ((MOD(K1,2) .EQ. 0) .AND.(MOD(K2,2) .EQ. 0) .AND.
     +  (MOD(K3,2) .EQ. 0)) THEN
        K1=K1/2
        K2=K2/2
        K3=K3/2
      ENDIF
      WRITE (WRLINE,'(A,I3,A,3I2,A,3F8.4,A,3F8.4)')
CYENG1
     + 'Typ:',ITT,' Achse:',K1,K2,K3,' P:',T,' Hub:',X2
CYENG2
CYENGY     + 'Type:',ITT,' Axis:',K1,K2,K3,' Pt:',T,' Gld:',X2
  100 CALL KPWRTE (NTOUT)
C
  999 RETURN
      END
