C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CZVTFHN   55             -------------------
                          SUBROUTINE ZVTFHN
C  CY-ENG-               -------------------
C
C  Das Progamm verwaltet die anisotropen Temperaturfaktoren.
C  NTYP=1: Eingabe   (TF)
C  NTYP=2: Loeschen  (LTF)
C  NTYP=3: Listen    (OTF)
C  NTYP=4: Erzeugen  (GTF)
C  NTYP=5: Temeraturfaktor auf AIN bringen
C  NTYP=6: Konvertiere Radien in Temperaturfaktoren (CVRT)
C  NTYP=7: Eingabe (TL)
C
C                  Geschrieben: 150829 JAN 1985 (Hundt)
C                  Geaendert:   071129 FEB 2000 (Hundt)
C
C                        -------------------
C
      IMPLICIT NONE
C
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
      INCLUDE 'KPORC.INC'
C
      INTEGER *4 INBUF(2),IAIN(72),IT1,IT2,I,IFLAG,II,J,K,L,M,NTYPS
      INTEGER *4 IALLE
      REAL    *4 Y(3,3),ANBUF(2)
C
      EQUIVALENCE (Y(1,1),X(1,4)),(INBUF(1),ANBUF(1)),(AIN(1),IAIN(1))
C
      DATA IALLE/4HALLE/
C
      NTYPS=NTYP
      GO TO (10,160,240,360,420,440,10), NTYPS
C             1   2   3   4   5   6  7
C
C                            Eingabe der Temperaturfaktoren
   10 DO  I=3,9
        AIN(I)=0.
      ENDDO
      IAIN(1)=ISKP(24)
      IAIN(2)=ISKP(24)
      IF (NTYPS .EQ. 7) THEN
        CALL FLES (AIN(3),ISKP(31),3H7FE,
CYENG1
     +  29Hu11,u22,u33,u12,u13,u23,Typ?$)
CYENG2
CYENGY     +  30Hu11,u22,u33,u12,u13,u23,type?$)
      ELSE
        CALL FLES (AIN,ISKP(31),5HAA7FE,
CYENG1
     +  41HSymb.,Untr.,u11,u22,u33,u12,u13,u23,Typ?$)
CYENG2
CYENGY     +  40HName,ext.,u11,u22,u33,u12,u13,u23,type?$)
        IF (ISKP(31) .LE. 0) THEN
          IERR=55 0054 16
          GO TO 999
        ENDIF
      ENDIF
      ISKP(14)=-1
      CALL ZVTFH2
      IF (NG .NE. 0) THEN
        CALL FLES (AIN,ISKP(31),1HK,2H?$)
        WRITE (WRLINE,'(A)')
CYENG1
     +  'ZVTFHN0064 - TF nicht positiv definit.'
CYENG2
CYENGY     +  'ZVTFHN0064 - TF not positiv definite.'
        CALL KPWRTE (NTDISP)
        IERR=-1
        GO TO 999
      ENDIF
      IFLAG=0
      IF (NTYPS .NE. 7 .OR. NAT(NS) .EQ. 0) GO TO 40
      DO  K=1,3
        DO  J=1,3
          TFVKT(J,K,NAT(NS),NS)=PA(J,K)*EV(K)
        ENDDO
      ENDDO
      IPFLG(NAT(NS),NS)=8
      IFLAG=1
      GO TO 45
C
C                            Suche.
   40 IFLAG=0
      DO  I=ISKP(53),NAT(NS)
        IF (AT(5,I,NS) .EQ. AIN(1) .AND. AT(6,I,NS) .EQ. AIN(2)) THEN
          DO  K=1,3
            DO  J=1,3
              TFVKT(J,K,I,NS)=PA(J,K)*EV(K)
            ENDDO
          ENDDO
          IPFLG(I,NS)=8
          IFLAG=1
        ENDIF
      ENDDO
   45 IF (IFLAG .NE. 0) GO TO 999
   50 WRITE (WRLINE,'(A,2A4)')
CYENG1
     + 'ZVTFHN0098 - Kein passendes Atom gefunden zu ',AIN(1),AIN(2)
CYENG2
CYENGY     + 'ZVTFHN0098 - No matching atom found for ',AIN(1),AIN(2)
      CALL KPWRTE (NTDISP)
      GO TO 999
C                            Gezielt loeschen
  160 IAIN(1)=ISKP(24)
      IAIN(2)=ISKP(24)
CYENG1
      CALL FLES (AIN,ISKP(31),3HAAE,13HSymb.,Untr.?$)
CYENG2
CYENGY      CALL FLES (AIN,ISKP(31),3HAAE,11HName,ext.?$)
      AIN(3)=AIN(1)
      CALL CNVTUP (AIN(3),1,4)
      IF (IAIN(3) .EQ. IALLE) THEN
C
C                            Alles loeschen
        DO  I=ISKP(53),NAT(NS)
          IPFLG(I,NS)=0
        ENDDO
        GO TO 999
      ENDIF
C                            Durchsuche Liste und
C                            entferne die gefundenen
      DO  I=ISKP(53),NAT(NS)
        IF (AT(5,I,NS) .EQ. AIN(1)) THEN
          IF (IAIN(2) .EQ. ISKP(24) .OR. AIN(2) .EQ. AT(6,I,NS))
     +    IPFLG(I,NS)=0
        ENDIF
      ENDDO
      GO TO 999
C                            TF's listen
  240 INBUF(1)=ISKP(53)
      INBUF(2)=NAT(NS)
CYENG1
      CALL FLES (ANBUF,ISKP(31),3H2IE,9Hvon,bis?$)
CYENG2
CYENGY      CALL FLES (ANBUF,ISKP(31),3H2IE,9Hfrom,to?$)
      WRITE (WRLINE,321)
      CALL KPWRTE (NTOUT)
CYENG1
  321 FORMAT (' Nr   Name      U11      U22      U33      U12      U13',
     1 '      U23    Typ')
CYENG2
CYENGY  321 FORMAT (' No   Name      U11      U22      U33      U12      U13',
CYENGY     1 '      U23    Type')
      II=INBUF(1)-1
      GO TO 243
  241 II=II+1
        IF (IPFLG(II,NS) .LE. 0) GO TO 243
C
C                            Konvertiere ins kartesische system.
        DO  K=1,3
          DO  J=1,3
            PA(J,K)=DG(J,1,NS)*TFVKT(1,K,II,NS)+DG(J,2,NS)*
     +      TFVKT(2,K,II,NS)+DG(J,3,NS)*TFVKT(3,K,II,NS)
          ENDDO
        ENDDO
        DO  K=1,3
          DO  J=1,3
            S(K,J)=PA(K,1)*PA(J,1)+PA(K,2)*PA(J,2)+PA(K,3)*PA(J,3)
          ENDDO
        ENDDO
        GO TO 370
  242   WRITE (WRLINE,'(I4,A,2A4,6F9.6,I3)') II,') ',AT(5,II,NS),
     +  AT(6,II,NS),(S(J,J),J=1,3),S(1,2),S(1,3),S(2,3),IPFLG(II,NS)
        CALL KPWRTE (NTOUT)
  243 IF (II .LT. INBUF(2)) GO TO 241
      GO TO 999
C
C                            GTF - Generiere Temperaturfaktor
C
CYENG1
  360 CALL FLES (AIN,ISKP(31),5HAA3FE,25HName,Untersch.,d1,d2,d3?$)
CYENG2
CYENGY  360 CALL FLES (AIN,ISKP(31),5HAA3FE,20HName,ext.,d1,d2,d3?$)
C
C                       Baue Matrizen auf
      DO  I=ISKP(53),NAT(NS)
        IF (AIN(1) .EQ. AT(5,I,NS) .AND. AIN(2) .EQ. AT(6,I,NS)) THEN
          DO  J=1,3
            DO  K=1,3
              TFVKT(J,K,I,NS)=TRAINV(J,K,NS)*AIN(K+2)
            ENDDO
          ENDDO
          IPFLG(I,NS)=8
          GO TO 999
        ENDIF
      ENDDO
      GO TO 50
C                       Temperaturfaktor auf AIN bringen.
C
  420 IF (IPFLG(IANR,NS) .EQ. 0) THEN
        AIN(5)=0.
        GO TO 999
      ENDIF
      DO  M=1,3
        K=1
        DO  L=1,3
          Y(L,M)=SYM(K+1,ISYNR,NS)*TFVKT(1,M,IANR,NS)+
     +    SYM(K+2,ISYNR,NS)*TFVKT(2,M,IANR,NS)+
     +    SYM(K+3,ISYNR,NS)*TFVKT(3,M,IANR,NS)
          K=K+4
        ENDDO
      ENDDO
C                          Konvertiere ins kartesische system.
      DO  K=1,3
        DO  J=1,3
          PA(J,K)=DG(J,1,NS)*Y(1,K)+DG(J,2,NS)*Y(2,K)+DG(J,3,NS)*Y(3,K)
        ENDDO
      ENDDO
      DO  K=1,3
        DO  J=1,3
          S(K,J)=PA(K,1)*PA(J,1)+PA(K,2)*PA(J,2)+PA(K,3)*PA(J,3)
        ENDDO
      ENDDO
  370 DO  I=1,3
        DO  J=1,3
          Y(I,J)=RG(1,I,NS)*S(1,J)+RG(2,I,NS)*S(2,J)+RG(3,I,NS)*S(3,J)
        ENDDO
      ENDDO
      DO  I=1,3
        DO  J=1,3
          S(I,J)=(Y(I,1)*RG(1,J,NS)+Y(I,2)*RG(2,J,NS)+Y(I,3)*RG(3,J,NS))
     +    /(RK(I,NS)*RK(J,NS))
        ENDDO
      ENDDO
      IF (NTYPS .EQ. 3) GO TO 242
      IF (NTYPS .EQ. 5) GO TO 430
      DO  I=1,3
        AIN(I+2)=S(I,I)
      ENDDO
      AIN(6)=S(1,2)
      AIN(7)=S(1,3)
      AIN(8)=S(2,3)
      AIN(9)=8.
      GO TO 40
C
  430 DO  J=1,3
        AIN(J+5)=S(J,J)
      ENDDO
      AIN(9)=S(1,2)
      AIN(10)=S(1,3)
      AIN(11)=S(2,3)
      AIN(5)=1.
      GO TO 999
C                            CVRT - Konvertiere Radien in Tem.-fkt.
C
  440 IF (ISKP(14) .EQ. 0) THEN
        IAIN(1)=ISKP(53)
        IAIN(2)=NAT(NS)
CYENG1
        CALL FLES (AIN,ISKP(31),3H2IE,9Hvon,bis?$)
CYENG2
CYENGY        CALL FLES (AIN,ISKP(31),3H2IE,9Hfrom,to?$)
      ELSE
        ISKP(14)=0
      ENDIF
      IF (IAIN(1) .LE. 0 .OR. IAIN(2) .GT. NAT(NS) .OR.
     +  IAIN(1) .GT. IAIN(2)) THEN
        IERR=58 0258 16
        GO TO 999
      ENDIF
      IT1=IAIN(1)
      IT2=IAIN(2)
      DO  K=IT1,IT2
        IF (IPFLG(K,NS) .EQ. 0) THEN
          DO  I=4,8
            AIN(I)=0.
          ENDDO
          AIN(3)=AT(4,K,NS)/CONST(43)
          AIN(9)=7.
          ISKP(14)=1
          CALL ZVTFH2
          IF (NG .NE. 0) THEN
            WRITE (WRLINE,'(A,I5,A)')
CYENG1
     +      'ZVTFHN0273 - Atom',K,' nicht positiv definit.'
CYENG2
CYENGY     +      'ZVTFHN0273 - Atom',K,' not positiv definite.'
            CALL KPWRTE (NTDISP)
          ELSE
            DO  I=1,3
              DO  J=1,3
                TFVKT(J,I,K,NS)=PA(J,I)*EV(I)
              ENDDO
            ENDDO
            IPFLG(K,NS)=8
          ENDIF
        ENDIF
      ENDDO
C
  999 RETURN
      END
