C KPLOT - A Program for Plotting and Analyzing Crystal Structures 
C Copyright (C) 1979-2019 Rudolf Hundt 
C KPLOT is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
C as published by the Free Software Foundation, version 3.
C KPLOT is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
C without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
C See the GNU General Public License for more details.
C You should have received a copy of the GNU General Public License
C along with this program.  If not, see https://www.gnu.org/licenses/
C Contact: Rudolf Hundt, University of Bonn Klaus Pleuger,
C Co-Programmer, mailto:k.pleuger@t-online.de
C
CKPZ      20               ----------------
                            SUBROUTINE ZVZ
C  CY-ENG-                 ----------------
C
C                       Geschrieben: Hundt
C                Geaendert: 261214 MAR 2018  (Hundt)
C
C                Verarbeitung der Gitterkonstanten.
C
C                          ----------------
      IMPLICIT NONE
C
      INCLUDE 'ZVCOM.INC'
      INCLUDE 'WRBFR.INC'
      INCLUDE 'KPORC.INC'
C
      CHARACTER *40 SVLINE
      REAL    *4 ST(4,4),STINV(4,4),ZELLSV(7),STHILF(4,4),ANULL
      REAL    *4 SZBUF(6),SZTOL(6),ARCCOS,TEMP,AIG(3)
      REAL    *4 TRABIJ(3,3),ORGSH(3),A(3,3),TIVBIJ(3,3)
      INTEGER *4 IAIN(72),ISZLP(2,3),IOPT,NTYPS
      INTEGER *4 I,J,K,L,NRMRET,NATS,KEYIN,ISKP58,ITZC,NSYMSV
      INTEGER *4 IAT(8,PARDIM,2)
C
      SAVE ZELLSV,SZBUF,SZTOL,TRABIJ,ORGSH,TIVBIJ
C
      EQUIVALENCE (AIN(1),IAIN(1)),(AT(1,1,1),IAT(1,1,1))
      EQUIVALENCE (AIN(19),ST(1,1)),(AIN(35),STINV(1,1))
      EQUIVALENCE (AIN(51),STHILF(1,1))
C
      DATA ANULL/4HNull/,AIG/1Ha,1Hb,1Hc/
C
C     1:  Z    - Eingabe der Zelle
C     2:  TZ   - Transformiere Zelle
C     3:  KS   - Jetzt in ZVK
C     4:  TZUR - Transformiere Zellursprung
C     5:  RZ   - Restore Zelle
C     6:  ZTAK - Zelle und Transformation der Atomkoordinaten
C     7:  XZ   - Zelle und reziproke Zelle vertauschen
C     8:  SZ   - Zelle suchen
C     9:  TZP  - Transformiere Zelle nach Parametern
C    10:  SZAT - Suche Zelle automatisch - Toleranzen
C    11:  SZA  - Suche Zelle automatisch
C    12:  TZUP - Transformiere Zellursprung auf Punkt
C    13:         Initialisierung
C    14:  OTM  - Gib Gesamttransformation aus
C    15:  SZXZ - Vertausche Suchzelle mit eingegebener
C    16:         Reset TRABIJ und ORGSH
C    17:  TZC  - Transformiere Zelle und fuehre ggf. Zentrierungen ein
C    18:  TZPC - Transformiere Zelle nach Parametern
C    19:         Ausgabe der HKLF 4 Zeile
C    20:  ZBAS - Zelle durch Eingabe der Basis des direkten Gitters
C    21:         Kopiere TRABIJ und ORGSH auf S und T
C    22:         Kopiert TRABIJ auf AIN
C
      NTYPS=NTYP
      ITZC=0
      GO TO (20,300,999,580,770,10,790,810,300,900,
C            1  2   3   4   5   6  7   8   9   10
     + 910,571,920,930,940,935,260,300,950,960,936,970),NTYP
C      11  12  13  14  15  16   17  18  19  20  21  22
C
C                            Definition der Zelle.  Setze Defaults.
   10 IOPT=1
      DO  I=ISKP(53),NAT(NS)
        IPFLG(I,NS)=0
      ENDDO
      GO TO 30
   20 IOPT=0
C                            Save alte Zelle.
   30 CONTINUE
      DO  I=1,7
        ZELLSV(I)=ZELLE(I,NS)
      ENDDO
      IF (ISKP(14) .NE. 0) GO TO 60
      CALL FLES (ZELLE(1,NS),ISKP(31),4H6FE ,
     + 24Ha,b,c,alpha,beta,gamma?$)
C
C                            Teste auf Gueltigkeit.
C
   60 IF (ZELLE(1,NS) .LE. 0.) GO TO 990
C
C                            Setze ggf. Defaults.
C
      IF (ZELLE(2,NS) .LE. 0.) ZELLE(2,NS)=ZELLE(1,NS)
      IF (ZELLE(2,NS) .EQ. 1.) ZELLE(2,NS)=ZELLE(1,NS)
      IF (ZELLE(3,NS) .LE. 0.) ZELLE(3,NS)=ZELLE(1,NS)
      IF (ZELLE(3,NS) .EQ. 1.) ZELLE(3,NS)=ZELLE(1,NS)
      DO  I=4,6
        IF (ZELLE(I,NS) .LE. 0.) ZELLE(I,NS)=90.
      ENDDO
C                            Teste Plausibilitaet der Winkel.
      DO  I=1,3
        T(I)=ZELLE(I+3,NS)
        IF (T(I) .LT. 5. .OR. T(I) .GT. 175.) GO TO 70
      ENDDO
C                            Auch die Winkelsumme sollte
C                            360 Grad nicht ueberschreiten.
C
      IF (T(1)+T(2)+T(3) .LT. 355. .AND.
     +    T(1)-T(2)-T(3) .LT. -5. .AND.
     +   -T(1)+T(2)-T(3) .LT. -5. .AND.
     +   -T(1)-T(2)+T(3) .LT. -5.) GO TO 80
   70 IERR=20 0104 10
      GO TO 999
C
C                            Basis des direkten Gitters.
   80 DO  I=1,3
        DO  J=1,3
          S(I,J)=DG(I,J,NS)
          DG(I,J,NS)=0.
        ENDDO
      ENDDO
      DG(1,1,NS)=ZELLE(1,NS)
      DG(1,2,NS)=ZELLE(2,NS)*COS(ZELLE(6,NS)*CONST(3))
      DG(2,2,NS)=SQRT(ZELLE(2,NS)*ZELLE(2,NS)-DG(1,2,NS)*DG(1,2,NS))
      DG(1,3,NS)=ZELLE(3,NS)*COS(ZELLE(5,NS)*CONST(3))
      DG(2,3,NS)=(ZELLE(2,NS)*ZELLE(3,NS)*COS(ZELLE(4,NS)*CONST(3))-
     1 DG(1,2,NS)*DG(1,3,NS)) / DG(2,2,NS)
      DG(3,3,NS)=SQRT(ZELLE(3,NS)*ZELLE(3,NS)-DG(1,3,NS)*DG(1,3,NS)-
     1 DG(2,3,NS)*DG(2,3,NS))
C
C                            Metrischer Tensor
  140 CONTINUE
      DO  I=1,3
        DO  J=1,3
          AA(I,J,NS)=DG(1,I,NS)*DG(1,J,NS)+DG(2,I,NS)*DG(2,J,NS)+
     +    DG(3,I,NS)*DG(3,J,NS)
          BB(I,J,NS)=AA(I,J,NS)
          RG(I,J,NS)=DG(J,I,NS)
        ENDDO
      ENDDO
      CALL MINV (BB(1,1,NS),3,TEMP,AIN(11),AIN(14))
C
C              Basis des reziproken Gitters und Zellvolumen
C
      CALL MINV (RG(1,1,NS),3,ZELLE(7,NS),AIN(11),AIN(14))
C
C                            Transformiere optional Koordinaten.
C
      IF ((IOPT .NE. 0) .AND. (NAT(NS) .GE. ISKP(53))) THEN
        DO  I=1,3
          DO  J=1,3
            X(I,J)=RG(1,I,NS)*S(1,J)+RG(2,I,NS)*S(2,J)+RG(3,I,NS)*S(3,J)
          ENDDO
        ENDDO
        DO  I=ISKP(53),NAT(NS)
          DO  J=1,3
            T(J)=AT(J,I,NS)
          ENDDO
          DO  J=1,3
            AT(J,I,NS)=X(J,1)*T(1)+X(J,2)*T(2)+X(J,3)*T(3)
          ENDDO
        ENDDO
      ENDIF
C                            Bestimme rez. Gitterkonstanten.
      DO  I=1,3
        RK(I,NS)=SQRT(RG(1,I,NS)*RG(1,I,NS)+RG(2,I,NS)*RG(2,I,NS)+
     +  RG(3,I,NS)*RG(3,I,NS))
      ENDDO
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        RK(K+3,NS)=ARCCOS((RG(1,I,NS)*RG(1,J,NS)+RG(2,I,NS)*RG(2,J,NS)+
     1  RG(3,I,NS)*RG(3,J,NS))/(RK(I,NS)*RK(J,NS)))
      ENDDO
      ISYPOI=0
      IZEPOI=0
C                            Setze bei Transformationsmatrizen
C                            Defaults.
  250 I=NTYP
      NTYP=3
      CALL ZVK
      NTYP=I
      DO  I=1,2
        K8(I)=0
        K9(I)=0
      ENDDO
      IF (ISKP(14) .NE. 0) GO TO 999
      IF ((NTYP .LE. 2).OR.(NTYP .EQ. 6).OR.(NTYP .EQ. 20)) GO TO 999
      CALL FLES (AIN,ISKP(31),1HE,2H?$)
      GO TO 999
C                            TZC - TZ + automatisch C
  260 ITZC=1
      NTYP=2
C                            TZ, TZP - Transformiere Zelle
  300 NHMSNR(1,NS)=iskp(24)
      NHMSNR(2,NS)=iskp(24)
      NHMSNR(3,NS)=0
      ISYPOI=0
      IZEPOI=0
      ISKP58=0
      IF (NTYP .EQ. 18) THEN
        ITZC=1
        NTYP=9
      ENDIF
      IF (NTYP .EQ. 9) THEN
        IF (ISKP(14) .EQ. 0) THEN
          CALL FLES (ICDS,ISKP(31),3H3CE,10Hc1,c2,c3?$)
          ISKP58=NTIN
        ENDIF
        K=0
        DO  NC=1,3
          IF (ICDS(NC) .EQ. 0) THEN
            ICDS(NC)=NC
            NTYP=7
            CALL ZVGC01 (ICDS(NC))
          ENDIF
          NTRY=24
          CALL ZVDEK
          IF (IERR .NE. 0) GO TO 999
          DO  J=1,3
            K=K+1
            AIN(K)=X(J,NC)
          ENDDO
        ENDDO
	NTYP=2
      ELSE
        IF (ISKP(14) .EQ. 0) THEN
          CALL FLES (AIN,ISKP(31),3H9FE,19Hs11,s21, ..., s33?$)
          IF (ISKP(31) .LT. 9) THEN
            IERR=20 0222 16
            GO TO 999
          ENDIF
          ISKP58=NTIN
        ENDIF
      ENDIF
C                            Kopiere Eingabematrix ...
      K=0
      ST(1,1)=1.
      DO  I=2,4
        ST(I,1)=0.
        ST(1,I)=0.
        DO  J=2,4
          K=K+1
          ST(J,I)=AIN(K)
        ENDDO
      ENDDO
      DO  I=1,4
        DO  J=1,4
          STINV(I,J)=ST(I,J)
        ENDDO
      ENDDO
C                            ... und invertiere.
C
      CALL MINV (STINV,4,TEMP,AIN(11),AIN(15))
      IF (TEMP .LT. 0.) THEN
        WRITE (WRLINE,'(A)')
CYENG1
     +  'Transformation ergibt Linkssystem.  Abgelehnt.'
CYENG2
CYENGY     +  'Transformation results in a left handed system.  Denied.'
        CALL KPWRTE (NTDISP)
        GO TO 999
      ENDIF
      IF (TEMP .LT. 0.0001) THEN
C
C                            Matrix singulaer.
        IERR=20 0259 06
        GO TO 999
      ENDIF
C                            Drucke Matrizen
C
      IF (ISKP(58) .GT. 0 .OR. ISKP58 .EQ. 5) THEN
        WRITE (WRLINE,'(6X,A,22X,A)')
     +  'Transformationsmatrix','Inverse Matrix'
        CALL KPWRTE (NTOUT)
        DO  J=2,4
          WRITE (WRLINE,'(3F10.5,10X,3F10.5)')
     +    (ST(J,I),I=2,4),(STINV(J,I),I=2,4)
          CALL KPWRTE (NTOUT)
        ENDDO
      ENDIF
      IF (ISKP(14) .LT. 0) GO TO 310
C
C                            Update (1) origin ...
      DO  I=2,4
        T(I-1)=STINV(I,2)*ORGSH(1)+STINV(I,3)*ORGSH(2)+
     +  STINV(I,4)*ORGSH(3)
      ENDDO
      DO  I=1,3
        ORGSH(I)=T(I)
      ENDDO
C                            ... und (2) gesamtmatrix
      DO  I=1,3
        DO  J=1,3
          TIVBIJ(I,J)=TRABIJ(I,1)*ST(2,J+1)+TRABIJ(I,2)*ST(3,J+1)+
     +    TRABIJ(I,3)*ST(4,J+1)
        ENDDO
      ENDDO
      DO  I=1,3
        DO  J=1,3
          TRABIJ(I,J)=TIVBIJ(I,J)
        ENDDO
      ENDDO
      CALL MINV (TIVBIJ,3,S(1,1),S(1,2),S(1,3))
C
C                            Transformiere.
      DO  I=1,6
        ZELLSV(I)=ZELLE(I,NS)
      ENDDO
      DO  I=1,3
        DO  J=2,4
          S(I,J-1)=DG(I,1,NS)*ST(2,J)+DG(I,2,NS)*ST(3,J)+
     +    DG(I,3,NS)*ST(4,J)
          A(I,J-1)=S(I,J-1)
        ENDDO
      ENDDO
C                            Bestimme die neuen Gitterkonstanten.
C                            Berechne Skalarprodukte.
  305 DO  I=1,3
        DO  J=I,3
          DG(I,J,NS)=S(1,I)*S(1,J)+S(2,I)*S(2,J)+S(3,I)*S(3,J)
        ENDDO
      ENDDO
      DG(3,1,NS)=DG(1,3,NS)
C                            Berechne Achsenlaengen.
      DO  I=1,3
        ZELLE(I,NS)=SQRT(DG(I,I,NS))
      ENDDO
C                            Berechne Winkel.
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        ZELLE(K+3,NS)=ARCCOS(DG(I,J,NS)/(ZELLE(I,NS)*ZELLE(J,NS)))
      ENDDO
      DO  I=1,3
        DO  J=1,3
          DG(J,I,NS)=A(J,I)
        ENDDO
      ENDDO
      IF (NTYP .EQ. 20) GO TO 140
C
C                            Transformiere Ursprungspunkt.
      DO  I=2,4
        T(I-1)=STINV(I,2)*UR(1,NS)+STINV(I,3)*UR(2,NS)+
     +  STINV(I,4)*UR(3,NS)
      ENDDO
      DO  I=1,3
        UR(I,NS)=T(I)
      ENDDO
C
C                 Transformiere Koordinaten in der Atomliste ...
C
      IF (NAT(NS) .GE. ISKP(53)) THEN
        DO  K=ISKP(53),NAT(NS)
          DO  I=2,4
            T(I-1)=STINV(I,2)*AT(1,K,NS)+STINV(I,3)*AT(2,K,NS)+
     +      STINV(I,4)*AT(3,K,NS)
          ENDDO
          DO  I=1,3
            AT(I,K,NS)=T(I)
          ENDDO
C                            ... und ggf. Auslenkungsparameter.
C
          IF (IPFLG(K,NS) .EQ. 8) THEN
            DO  J=1,3
              DO  I=2,4
                T(I-1)=STINV(I,2)*TFVKT(1,J,K,NS)+
     +          STINV(I,3)*TFVKT(2,J,K,NS)+STINV(I,4)*TFVKT(3,J,K,NS)
              ENDDO
              DO  I=1,3
                TFVKT(I,J,K,NS)=T(I)
              ENDDO
            ENDDO
          ENDIF
        ENDDO
      ENDIF
C                            Transformiere Symmetrien
C
  310 IF (NSYM(NS) .LE. 0) GO TO 140
      DO  I=1,NSYM(NS)
        DO  J=1,4
          DO  K=2,4
          STHILF(K,J)=STINV(K,2)*SYM(J,I,NS)
     1               +STINV(K,3)*SYM(J+4,I,NS)+STINV(K,4)*SYM(J+8,I,NS)
          ENDDO
        ENDDO
        L=1
        DO  J=2,4
          DO  K=1,4
            SYM(L,I,NS)=STHILF(J,1)*ST(1,K)+STHILF(J,2)*ST(2,K)
     1                 +STHILF(J,3)*ST(3,K)+STHILF(J,4)*ST(4,K)
            L=L+1
          ENDDO
        ENDDO
C
      ENDDO
      IF (ISKP(14) .GE. 0) THEN
        ASSIGN 140 TO NRMRET
      ELSE
        ASSIGN 999 TO NRMRET
      ENDIF
      GO TO 650
C                            TZUP - Transf. auf Punkt
C
  571 IF (ISKP(14) .EQ. 0) THEN
        ICDS(1)=0
        CALL FLES (ICDS,ISKP(31),2HCE,6HCODE?$)
      ENDIF
      IF (ICDS(1) .EQ. 0) THEN
        ICDS(1)=1
        NTYP=7
        CALL ZVGC01 (ICDS(1))
      ENDIF
      NC=1
      NTRY=24
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
      DO  I=1,3
        AIN(I)=X(I,1)
      ENDDO
      GO TO 581
C                            TZUR - Transformiere Zellursprung
C
  580 IF (ISKP(14) .EQ. 0) THEN
        DO  I=1,3
          AIN(I)=0.
        ENDDO
        CALL FLES (AIN,ISKP(31),3H3FE,7Hx,y,z?$)
      ENDIF
  581 NHMSNR(1,NS)=iskp(24)
      NHMSNR(2,NS)=iskp(24)
      NHMSNR(3,NS)=0
      ISYPOI=0
      IZEPOI=0
C                            Symmetrien
C
      IF (NSYM(NS) .LE. 0) GO TO 620
      DO  I=1,NSYM(NS)
        J=1
        DO  K=1,3
          SYM(J,I,NS)=SYM(J,I,NS)-AIN(K)+SYM(J+1,I,NS)*AIN(1)+
     1    SYM(J+2,I,NS)*AIN(2)+ SYM(J+3,I,NS)*AIN(3)
          J=J+4
        ENDDO
      ENDDO
      ASSIGN 620 TO NRMRET
      GO TO 650
C                            Origin
C
  620 IF (ISKP(14) .LT. 0) GO TO 999
      DO  I=1,3
        ORGSH(I)=ORGSH(I)-AIN(I)
      ENDDO
C                            Atome
C
      IF (NAT(NS) .GE. ISKP(53)) THEN
        DO  I=ISKP(53),NAT(NS)
          DO  K=1,3
            AT(K,I,NS)=AT(K,I,NS)-AIN(K)
          ENDDO
        ENDDO
      ENDIF
      GO TO 999
C                            Normiere Translationen der Symmetrien
  650 DO 750 I=1,NSYM(NS)
      DO 740 J=1,9,4
      TEMP=SYM(J,I,NS)
      GO TO 670
  660 TEMP=TEMP+1.
  670 IF (TEMP) 680,730,700
  680 IF (TEMP+0.001) 660,720,720
C
  690 TEMP=TEMP-1.
  700 IF (TEMP-1.) 710,720,690
  710 IF (TEMP-0.999) 730,720,720
  720 TEMP=0.
  730 SYM(J,I,NS)=TEMP
  740 CONTINUE
  750 CONTINUE
C                            Falls Zentrierungen eingefuehrt werden
C                            sollen, generiere sie.
      IF (ITZC .NE. 0) THEN
        DO  J=2,4
          DO  I=2,4
            TEMP=STINV(I,J)
            GO TO 1430
 1420       TEMP=TEMP+1.
 1430       IF (TEMP .LT. 0.) GO TO 1420
            GO TO 1450
 1440       TEMP=TEMP-1.
 1450       IF (TEMP .GE. 1.) GO TO 1440
            IF (TEMP .LT. 0.) TEMP=0.
            AIN(I-1)=TEMP
          ENDDO
          NSYMSV=NSYM(NS)
          ISKP(14)=1
          NTYP=2
          CALL ZVGRTS
          IF (IERR .NE. 0) THEN
            WRITE (WRLINE,'(A,A)') 'ZVZ0460 - ',
CYENG1
     +      'Kann Zentrierung nicht einfuehren.'
CYENG2
CYENGY     +      'Unable to introduce centering symmetry.'
            CALL KPWRTE (NTDISP)
            NSYM(NS)=NSYMSV
            GO TO 760
          ENDIF
          NSYMSV=NSYM(NS)
        ENDDO
      ENDIF
C
  760 IOPT=0
      GO TO NRMRET,(140,620,999)
C                            Restore Zelle
  770 DO 780 I=1,7
  780 ZELLE(I,NS)=ZELLSV(I)
  785 IOPT=0
      ISYPOI=0
      IZEPOI=0
      GO TO 60
C                            XZ - Zelle und reziproke umtauschen.
C                            Save alte Zelle.
  790 DO 800 I=1,7
      ZELLSV(I)=ZELLE(I,NS)
      ZELLE(I,NS)=RK(I,NS)
  800 CONTINUE
      GO TO 785
C                            SZ - Suche Zelle
C
  810 CALL FLES (AIN,ISKP(31),5H6I6FE,21Ha1,a2, ... ,ga1,ga2?$)
  811 DO  I=1,6,2
        IF ((IAIN(I) .GT. IAIN(I+1)) .OR.
     1     (IAIN(I) .LE. 0) .OR.
     2     (IAIN(I+1) .GT. NAT(NS)) .OR.
     3     (AIN(I+6) .GT. AIN(I+7)) .OR.
     4     (AIN(I+6) .LT. 1.) .OR.
     5     (AIN(I+7) .GT. 179.)) THEN
             IERR=20 0531 16
             GO TO 999
        ENDIF
      ENDDO
      AIN(31)=1.E6
C
      DO  I=1,6
        AIN(I+12)=COS(AIN(I+6)*CONST(3))
      ENDDO
C
      NTRY=26
      ICDS(1)=IAIN(1)-1
  830 NC=1
      ICDS(NC)=ICDS(NC)+1
      IF (ICDS(NC) .GT. IAIN(2)) THEN
        IF (NTYPS .EQ. 11 .AND. AIN(31) .LT. 1.E5) THEN
          AIN(31)=SQRT(AIN(31))
          WRITE (WRLINE,'(A,3I5,A,F7.2,A)')
CYENG1
     +    'Beste Loesung:',(IAIN(I),I=28,30),'  (',AIN(31),')'
CYENG2
CYENGY     +    'Best result:',(IAIN(I),I=28,30),'  (',AIN(31),')'
          CALL KPWRTE (NTOUT)
          DO  I=1,3
            DO  J=1,3
              X(J,I)=AT(J,IAIN(28),NS)*TIVBIJ(1,I)+
     +        AT(J,IAIN(29),NS)*TIVBIJ(2,I)+
     +        AT(J,IAIN(30),NS)*TIVBIJ(3,I)
            ENDDO
          ENDDO
        ENDIF
        GO TO 999
      ENDIF
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
      TEMP=0.
      DO  J=1,3
        TEMP=TEMP+X1(J)*X1(J)
        X(J,NC)=X1(J)
      ENDDO
      IF (TEMP .LE. 0.) GO TO 830
      AIN(19)=SQRT(TEMP)
C
      ICDS(2)=IAIN(3)-1
  850 NC=2
      ICDS(NC)=ICDS(NC)+1
      IF (ICDS(NC) .GT. IAIN(4)) GO TO 830
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
      TEMP=0.
      DO  J=1,3
        TEMP=TEMP+X1(J)*X1(J)
        X(J,NC)=X1(J)
      ENDDO
      IF (TEMP .LE. 0.)  GO TO 850
      AIN(20)=SQRT(TEMP)
C                           Teste ersten Winkel
      TEMP=AIN(19)*AIN(20)
      TEMP=(X(1,1)*X(1,2)+X(2,1)*X(2,2)+X(3,1)*X(3,2))/TEMP
      IF (TEMP .GT. AIN(17)) GO TO 850
      IF (TEMP .LT. AIN(18)) GO TO 850
      AIN(24)=ARCCOS(TEMP)
C
      ICDS(3)=IAIN(5)-1
  870 NC=3
      ICDS(NC)=ICDS(NC)+1
      IF (KEYIN(0) .EQ. ISKP(20)) GO TO 999
      IF (ICDS(NC) .GT. IAIN(6)) GO TO 850
      CALL ZVDEK
      IF (IERR .NE. 0) GO TO 999
      TEMP=0.
      DO  J=1,3
        TEMP=TEMP+X1(J)*X1(J)
        X(J,NC)=X1(J)
      ENDDO
      IF (TEMP .LE. 0.) GO TO 870
      AIN(21)=SQRT(TEMP)
C                         Teste restlichen Winkel
      TEMP=AIN(19)*AIN(21)
      TEMP=(X(1,1)*X(1,3)+X(2,1)*X(2,3)+X(3,1)*X(3,3))/TEMP
      IF (TEMP .GT. AIN(15)) GO TO 870
      IF (TEMP .LT. AIN(16)) GO TO 870
      AIN(23)=ARCCOS(TEMP)
C
      TEMP=AIN(20)*AIN(21)
      TEMP=(X(1,2)*X(1,3)+X(2,2)*X(2,3)+X(3,2)*X(3,3))/TEMP
      IF (TEMP .GT. AIN(13)) GO TO 870
      IF (TEMP .LT. AIN(14)) GO TO 870
      AIN(22)=ARCCOS(TEMP)
C                               Berechne Volumen
      TEMP=0.
      DO  I=1,3
        J=ID2(I)
        K=ID2(J)
        TEMP=TEMP+(X(J,1)*X(K,2)-X(K,1)*X(J,2))*X(I,3)
      ENDDO
C
      IF (NTYPS .NE. 11 .OR. TEMP .GT. 0 .OR. ISKP(58) .GT. 0) THEN
        WRITE (WRLINE,890) (ICDS(J),J=1,3),(AIN(J),J=22,24),TEMP
CYENG1
  890   FORMAT ('Loesung:',3I4,3F10.3,' Vol=',F10.2)
CYENG2
CYENGY  890   FORMAT ('Result:',3I4,3F10.3,' Vol=',F10.2)
        CALL KPWRTE (NTOUT)
        DO  I=1,3
          DO  J=1,3
            S(J,I)=AT(J,ICDS(1),NS)*TIVBIJ(1,I)+
     +      AT(J,ICDS(2),NS)*TIVBIJ(2,I)+
     +      AT(J,ICDS(3),NS)*TIVBIJ(3,I)
          ENDDO
        ENDDO
        WRITE (WRLINE,'(3(3F8.5,2X))') ((S(J,I),J=1,3),I=1,3)
        CALL KPWRTE (NTOUT)
C                            Speichere nur rechtshaendige loesungen
C
        IF (TEMP .GT. 0.) THEN
          TEMP=(AIN(19)-SZBUF(1))**2+(AIN(20)-SZBUF(2))**2+
     +    (AIN(21)-SZBUF(3))**2
          IF (TEMP .LT. AIN(31)) THEN
            DO  I=1,3
              IAIN(I+27)=ICDS(I)
            ENDDO
            AIN(31)=TEMP
          ENDIF
        ENDIF
      ENDIF
      GO TO 870
C                            SZAT - Definition der Toleranzen
C
  900 CALL FLES (SZTOL,ISKP(31),3H6FE,23Htola,tolb,..,tolgamma?$)
      IF (ISKP(31) .EQ. 0) THEN
        WRITE (WRLINE,'(6F10.5)') SZTOL
        CALL KPWRTE (NTOUT)
      ENDIF
      GO TO 999
C                            SZA - Suche Zelle automatisch
C
  910 IF (ISKP(14) .EQ. 0) THEN
        CALL FLES (SZBUF,ISKP(31),3H6FE,14Ha,b,..,gamma?$)
        IF (ISKP(31) .EQ. 0) THEN
          WRITE (WRLINE,'(3F10.5,3F10.3)') SZBUF
          CALL KPWRTE (NTOUT)
        ENDIF
      ENDIF
      NCD(NS)=0
      K8(NS)=0
      K9(NS)=0
      IF (NSYM(NS) .LE. 0) NSYM(NS)=1
      NATS=NAT(NS)
      IF (NAT(NS) .GE. NATDM) THEN
        IERR=20 0681 02
        GO TO 999
      ENDIF
      NAT(NS)=NAT(NS)+1
C                            Erzeuge Atom mit den Koord. 0,0,0
      DO  I=1,3
        AT(I,NAT(NS),NS)=0.
      ENDDO
      AT(4,NAT(NS),NS)=.3
      AT(5,NAT(NS),NS)=ANULL
      IAT(6,NAT(NS),NS)=ISKP(24)
      AT(7,NAT(NS),NS)=ANULL
C                            Stelle Kandidaten fuer die
C                            Gitterkonstanten zusammen.
      ISZLP(1,1)=NAT(NS)+1
      DO  J=1,3
C                            Setze Befehl 'AKS' auf.
        DO  I=1,3
          IAIN(I)=NATS+1
        ENDDO
        AIN(4)=SZBUF(J)+SZTOL(J)
        AIN(5)=SZBUF(J)-SZTOL(J)
        NTYP=3
        ISKP(14)=-3
        ISKP(31)=3
        ISKP(32)=2
        NCD(NS)=0
        K8(NS)=0
        K9(NS)=0
        CONST(20)=0.
        CALL ZVAVBR
        IF (IERR .NE. 0) GO TO 999
C
C                              Teste, ob etwas gefunden.
        IF (NCD(NS) .LE. 0) THEN
          WRITE (WRLINE,'(A,A1,A)')
CYENG1
     1     'Keinen Kandidaten fuer ',AIG(J),' gefunden.'
CYENG2
CYENGY     1     'No candidate found for ',AIG(J),'.'
          CALL KPWRTE (NTDISP)
          IERR=-2
          GO TO 999
        ENDIF
C                              Setze Befehl 'ACAL' auf.
        IAIN(1)=1
        IAIN(2)=NCD(NS)
        ISKP(14)=1
        NTYP=7
        CALL ZVGE03
        ISKP(14)=0
        IF (IERR .NE. 0) GO TO 999
        ISZLP(2,J)=NAT(NS)
        IF (J .LT. 3) ISZLP(1,J+1)=NAT(NS)+1
      ENDDO
C                            Benenne Atome um.
      DO  I=1,3
        DO  J=ISZLP(1,I),ISZLP(2,I)
          AT(5,J,NS)=AIG(I)
          AT(7,J,NS)=AIG(I)
          CALL CNVTUP (AT(7,J,NS),1,4)
        ENDDO
      ENDDO
C                            Jetzt kann gesucht werden.
      NCD(NS)=0
      K8(NS)=0
      K9(NS)=0
      J=0
      DO  I=1,3
        J=J+1
        IAIN(J)=ISZLP(1,I)
        AIN(J+6)=SZBUF(I+3)-SZTOL(I+3)
        J=J+1
        IAIN(J)=ISZLP(2,I)
        AIN(J+6)=SZBUF(I+3)+SZTOL(I+3)
      ENDDO
      GO TO 811
C                            INITIALISIERUNG
  920 DO  I=1,3
        ORGSH(I)=0
        ZELLSV(I)=1.
        ZELLSV(I+3)=90.
        SZTOL(I)=1.
        SZTOL(I+3)=4.
        SZBUF(I)=1.
        SZBUF(I+3)=90.
        DO  J=1,3
          TRABIJ(J,I)=0.
          TIVBIJ(J,I)=0.
        ENDDO
        TRABIJ(I,I)=1.
        TIVBIJ(I,I)=1.
      ENDDO
      GO TO 999
C                            Ausgabe der Transformationsmatrizen
  930 CONTINUE
CYENG1
      WRITE (WRLINE,'(A)') 'Bisher ausgefuehrte Transformationen:'
CYENG2
CYENGY      WRITE (WRLINE,'(A)') 'Transformations performed so far:'
      CALL KPWRTE (NTOUT)
      DO  I=1,3
        T(I)=-TRABIJ(I,1)*ORGSH(1)-TRABIJ(I,2)*ORGSH(2)
     +       -TRABIJ(I,3)*ORGSH(3)
        DO  J=1,3
          S(J,I)=TRABIJ(J,I)
        ENDDO
      ENDDO
      WRITE (WRLINE,'(A,3F10.6)') 'TZUR ',T
      CALL KPWRTE (NTOUT)
      WRITE (WRLINE,'(A,3(1X,3F8.5))') 'TZ',((TRABIJ(I,J),I=1,3),J=1,3)
      CALL KPWRTE (NTOUT)
C                            Inverse transformation
C
      WRITE (WRLINE,'(A,3F10.6,A)') ' (   ',ORGSH,')'
      CALL KPWRTE (NTOUT)
      CALL MINV (S,3,AIN(1),IAIN(2),IAIN(5))
      WRITE (WRLINE,'(A,3(1X,3F8.5),A)') ' (',((S(I,J),I=1,3),J=1,3),')'
      CALL KPWRTE (NTOUT)
      IAIN(1)=0
      CALL FLES (IAIN,ISKP(31),2HIE,14H[0]/1: Reset?$)
      IF (IAIN(1) .EQ. 0) GO TO 999
  935 DO  I=1,3
        DO  J=1,3
          TRABIJ(J,I)=0.
          TIVBIJ(J,I)=0.
        ENDDO
        TRABIJ(I,I)=1.
        TIVBIJ(I,I)=1.
        ORGSH(I)=0.
      ENDDO
      GO TO 999
C                            Kopiere TRABIJ und ORGSH auf S und T
  936 DO  I=1,3
        DO  J=1,3
          S(J,I)=TRABIJ(J,I)
        ENDDO
        T(I)=ORGSH(I)
      ENDDO
      GO TO 999
C                            SZXZ - Vertausche Suchzelle mit
C                            vorgegebener Zelle
C
  940 IF (ISKP(14) .EQ. 0) CALL FLES (AIN,ISKP(31),1HE,2H?$)
      DO  I=1,6
        TEMP=SZBUF(I)
        SZBUF(I)=ZELLE(I,NS)
        ZELLE(I,NS)=TEMP
      ENDDO
      IOPT=0
      NTYP=1
      GO TO 60
C                            HKLF 4
C
  950 WRITE (WRLINE,'(A)')
     1  'HKLF 4 1 '
      K=11
      DO  J=1,3
        DO  I=1,3
          TEMP=ABS(TRABIJ(I,J))
          L=9
          IF (ABS(TEMP) .LT. CONST(21)) L=4
          IF (ABS(TEMP-1.) .LT. CONST(21)) L=4
          IF (ABS(TEMP-0.5) .LT. CONST(21)) L=4
          IF (L .EQ. 9) THEN
            WRITE (WRLINE(K:K+8),'(F9.5)') TRABIJ(I,J)
          ELSE
            WRITE (WRLINE(K:K+3),'(F4.1)') TRABIJ(I,J)
          ENDIF
          K=K+L+1
        ENDDO
      ENDDO
      IF (K .GT. 80) THEN
        DO  I=80,1,-1
          IF (WRLINE(I:I) .EQ. ' ') GO TO 955
        ENDDO
  955   SVLINE=WRLINE(I+1:120)
        WRLINE(I:120)='= '
        CALL KPWRTE (NTPCH)
        WRITE (WRLINE,'(A,A)') '    ',SVLINE
      ENDIF
      CALL KPWRTE (NTPCH)
      GO TO 999
C                   ZBAS - Gitterkonstanten als basisvektoren
C
  960 IF (ISKP(14) .EQ. 0) THEN
        CALL FLES (S,ISKP(31),3H9FE,5Hbij?$)
      ENDIF
      IOPT=0
      ISKP(14)=0
      DO  I=1,3
        DO  J=1,3
          A(I,J)=S(I,J)
        ENDDO
      ENDDO
      GO TO 305
C                          Kopiere TRABIJ auf AIN
  970 K=1
      DO  I=1,3
        DO  J=1,3
          AIN(K)=TRABIJ(J,I)
          TRABIJ(J,I)=0.
          K=K+1
        ENDDO
        TRABIJ(I,I)=1.
      ENDDO
      GO TO 999
C
  990 IERR=20 0889 10
C
  999 RETURN
      END
