#include <qgl.h>
#include <qapplication.h>
#include <qlist.h>
#include <math.h>
#include <stdio.h>

/* OpenGL's GL_3D_COLOR feedback vertex format. */
typedef struct _Feedback3Dcolor {
  GLfloat x;
  GLfloat y;
  GLfloat z;
  GLfloat red;
  GLfloat green;
  GLfloat blue;
  GLfloat alpha;
} Feedback3Dcolor;

class Atom3dDisp {
public:
	int nr;
    float r;
    int color;
    GLuint glDisplayList;
};		

class Polygon3D {
public:
	int color;
	int muster;
	QList<int> atome;
    GLuint glDisplayList;
};

class Atom3d {
public:
    float x;
    float y;
    float z;
    int nr;
};

class Bond3d {
public:
    int p1;
    int p2;
    float xAngle;
    float yAngle;
    float length;
    float dm;
    int color;
    GLuint glDisplayList;
};

class CKPlot3D : public QGLWidget
{
   Q_OBJECT
public:
    CKPlot3D(QWidget *parent=0, const char *name=0 );
    void Animate(int t);
    void BeginDraw();
    void EndDraw();
    void setRotate3D(bool flag);
    void setSolidMode(bool flag);
    void setRenderMode(bool flag);
    void incrementZPos();
    void decrementZPos();
    void Reset();
    void outputPS(QString str);
    void setBufferedMode(bool flag);
    void changeBackgroundColor();
    void RenderPixmap(int n);
    void processDrawCommand(float *ar ,long int *funcIn);
signals:
    void keyPressed( QKeyEvent* );
protected:
    void initializeGL();
    void resizeGL( int, int );
public slots:
    void paintGL();
protected:
    void draw();
    void timerEvent( QTimerEvent * );
    void SetupGL();
    void setColorTable();
    Atom3d *getAtom(int nr);
    void mouseMoveEvent(QMouseEvent * e);
    virtual void mousePressEvent( QMouseEvent * );
    virtual void mouseReleaseEvent( QMouseEvent * );
    virtual void keyPressEvent( QKeyEvent * );
    virtual void keyReleaseEvent( QKeyEvent * );
private:
    QList<Atom3d> atomList;
    QList<Bond3d> bondList;
    QList<Atom3dDisp> atomListDisp;
    QList<Polygon3D> polygonList;
    float zPos;
    float zPosOrg;
    GLfloat angle;
    int solidFlag;
    float rotateStep;
    float zPosDiff;
    float xPosDiff;
    float yPosDiff;
//    GLfloat colors[16][4];
    int timer;
    int segments;
GLfloat pointSize;
QGLFormat *glFormat;
QImage * img1;
int aktRenderPixmap;
 public:
    void SetAtom3D(float x, float y,float z,long int  nr);
	void SetBond3D(long int p1, long int p2, long int color, float  radius);     
    void SetAtom3DDisp(long int nr, float r, long int color);
	void SetRotateAngles();
    void ResetDataModel();
public:
    // Button and key states
    typedef enum 
    { 
        DM_NONE,
        DM_ORBIT,
        DM_PAN,
        DM_ZOOM,
    } DragMode ;
    DragMode 		m_enDragMode ;
//    float 			m_vecCamPos[3] ;
//    float 			m_vecCenter[3] ;
    // Current viewport size
    long 			m_xsize ;
    long 			m_ysize ;

    // Mouse position when mouse button is pressed
    float 			m_lx ;
    float 			m_ly ;
    bool 			m_shift ;
private:
    // Constants
    static const float vecResetCamPos[3] ;
    static const float vecInitCenter[3] ;
void
print3DcolorVertex(GLint size, GLint * count,
  GLfloat * buffer);
void
printBuffer(GLint size, GLfloat * buffer);
GLfloat *
spewPrimitiveEPS(FILE * file, GLfloat * loc);

void
spewUnsortedFeedback(FILE * file, GLint size, GLfloat * buffer);
void
spewSortedFeedback(FILE * file, GLint size, GLfloat * buffer);
void
spewWireFrameEPS(FILE * file, int doSort, GLint size, GLfloat * buffer, char *creator);
void
outputEPS(int size, int doSort, char *filename);


};


